package atc.safefly.communication;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.math.BigInteger;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;

import org.apache.tools.ant.types.selectors.DateSelector;

import aglobex.simulation.global.ConversionConstants;
import atc.ontology.gpsflightplan.math.GpsPosition;
import atc.safefly.externaldata.integralog.NoDataInFileException;
import au.com.bytecode.opencsv.CSVReader;

public class Aligning {

	public final long MIN_OFFSET = -900*1000;
	public final long MAX_OFFSET = 900*1000;

	/**
	 * @param args
	 * @throws IOException
	 * @throws NumberFormatException
	 */

	public static void vypisGps(GpsPosition pos) {
		System.out.printf("altitude:" + pos.getAltitudeFT() + "[ft] latitude:" + pos.getLatitudeDeg() + "[deg] longitude:"
				+ pos.getLongitudeDeg() + "[deg]%n");
	}

	/**
	 * @param altitudeA
	 * @param altitudeB
	 * @return (altitudeA-AltitudeB)^2
	 */
	public static double diff(double altitudeA, double altitudeB) {
		return (double) (altitudeA - altitudeB) * (altitudeA - altitudeB);
	}

	public static double calculateIntegraZaonTimeOffset(Flight fl) {
		
		final long START_OFFSET = -900*1000;
		final long FINISH_OFFSET = 900*1000;
		double retVal = 0;
		int zaonLenght = fl.getZaonSimulationData().getZaonArray().length;
		int integraLength = fl.getCh7Gps().getCh7array().length;
		
		//guessing offsetTime
		String meanTimeIntegra = fl.getCh7Gps().getCh7array()[fl.getCh7Gps().getCh7array().length / 2].getTimeString();
		String meanTimeZaon = fl.getZaonSimulationData().getZaonArray()[fl.getZaonSimulationData().getZaonArray().length / 2]
				.getTimeString();
		Date dateZ = null;
		Date dateI = null;
		SimpleDateFormat sdf = new SimpleDateFormat("hhmmss");
		try {
			dateZ = sdf.parse(meanTimeZaon);// "000000");//
			dateI = sdf.parse(meanTimeIntegra); // "000100");//
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// odhad z poloviny intervalu dat
		long offsetTimeGuess = (dateZ.getTime() - dateI.getTime());
		// tested offset
		long offsetTesting;

		// max and minimal searched dates
		Date dateStart = new Date(dateI.getTime() + offsetTimeGuess + START_OFFSET );
		Date dateFinish = new Date(dateI.getTime() + offsetTimeGuess + FINISH_OFFSET);
		Calendar calStart = GregorianCalendar.getInstance(Locale.GERMAN);
		Calendar calFinish = GregorianCalendar.getInstance(Locale.GERMAN);
		calStart.setTime(dateStart);
		calFinish.setTime(dateFinish);
		// temporary Integra and Zaon dates - should be equal for one iteration
		Date dateTempI =null;
		Date dateTempZ =null;
		Calendar calTempI = GregorianCalendar.getInstance(Locale.GERMAN); // ITime + iterations + offset 
		Calendar calTempZ = GregorianCalendar.getInstance(Locale.GERMAN); // ZTime + iterations
		// date we try to find in Zaon and Integra simuateously
		Date dateTempBoth =null;
		Calendar calTempBoth = GregorianCalendar.getInstance(Locale.GERMAN);
		// date for 1 iteration of offset (adding 1 second at at time)
		Date dateIter = dateStart;
		Calendar calIter = GregorianCalendar.getInstance(Locale.GERMAN);
		calIter.setTime(dateIter);
		// indexes of Integra and Zaon arrays
		int indexIBase = fl.getCh7Gps().getCh7array().length/2;
		int indexZBase = fl.getZaonSimulationData().getZaonArray().length/2;
		int indexITemp = indexIBase;
		int indexZTemp = indexZBase;
		int indexIStartOfTestInterval;
		// sizes of datasets
		int sizeIntegraDataset = fl.getCh7Gps().getCh7array().length;
		int sizeZaonDataset	= fl.getZaonSimulationData().getZaonArray().length;
		//
		int numberOfSamples =0;
		double comparsionTesting =0;
		double comparsionBest = Double.MAX_VALUE;
		long offsetBest =0;
		
		offsetTesting = offsetTimeGuess + START_OFFSET;
		
		do {
			try {
				dateTempI = sdf.parse(fl.getCh7Gps().getCh7array()[indexITemp].getTimeString());
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			calTempI.setTime(dateTempI);
			calIter.setTime(dateTempI);
			indexITemp--; //TODO osetrit vyteceni ven
		} while (calTempI.after(calStart));
		indexIStartOfTestInterval = indexITemp; // mam zacatek testovani
		
		// cyklus pro ruzne offsety
		do {
			
			//cyklus na zkontrolovani jednoho offsetu
			do {
				calTempBoth.setTime(dateStart); //reference 
				try {
					dateTempI =sdf.parse(fl.getCh7Gps().getCh7array()[indexITemp].getTimeString());
					dateTempZ = sdf.parse(fl.getZaonSimulationData().getZaonArray()[indexZTemp].getTimeString());
				} catch (ParseException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				calTempI.setTime(dateTempI);
				calTempZ.setTime(dateTempZ);
				calTempI.add(Calendar.SECOND, (int)offsetTesting/1000);
				//taking care of index out of bound
				if (indexITemp >= sizeIntegraDataset || indexZTemp >= sizeZaonDataset) {
					//TODO vyskocit a nepocitat
					comparsionTesting = Double.MAX_VALUE;
					break;
				}
				if (calTempI.before(calTempBoth)) {
					indexITemp++;
					continue;
				}
				if (calTempZ.before(calTempBoth)){
					indexZTemp++;
					continue;
				}
				if (calTempI.after(calTempBoth) || calTempZ.after(calTempBoth)) {
					calTempBoth.add(Calendar.SECOND, 1);
					continue;
				}
				if (calTempI.compareTo(calTempBoth) == 0 && calTempZ.compareTo(calTempBoth) == 0) {
					comparsionTesting += diff(fl.getCh7Gps().getCh7array()[indexITemp].getAltitudeFT(), fl.getZaonSimulationData().getZaonArray()[indexZTemp].getPressureAltitudeFT());
					numberOfSamples++;
					calTempBoth.add(Calendar.SECOND, 1);
				}
				System.out.println("HOW DID I GOT HERE ?!");
			} while (numberOfSamples < 150); // 150 is for 5 minutes approx.
			
			if (comparsionTesting < comparsionBest) {
				comparsionBest = comparsionTesting;
				offsetBest = offsetTesting;
			}
			
			offsetTesting += 1000; // adding one second 
		} while (false);//konec cyklu pro ruzne offsety
		
		return retVal;
	} //end of calculateIntegraZaonTimeOffset

	public static void main(String[] args) {

		// init cele struktufy Flight
		// String integraFilename = "test_zaon_integra/1367937999";
		// String zaonFilename = "test_zaon_integra/zaon_log(1)raw_fix.txt";
		String integraFilename = "test_zaon_integra/integra2_R";
		String zaonFilename = "test_zaon_integra/zaon_log(2)raw_data_R.txt";
		Flight fl = new Flight(integraFilename, zaonFilename);

		// vypis GPS z Integry
		// System.out.println("fl.getTimestampMin() " + fl.getTimestampMin());
		// vypisGps(fl.getGpsPosition(fl.getTimestampMin()));
		// System.out.println("fl.getTimestampMax() " + fl.getTimestampMax());
		// vypisGps(fl.getGpsPosition(fl.getTimestampMax()));
		// System.out.println();
		// altitude Integra FT
		System.out.println("fl.getCh7Gps().getCh7array()[0].getAltitudeFT() " + fl.getCh7Gps().getCh7array()[0].getAltitudeFT());
		for (int i = 0; i < fl.getCh7Gps().getCh7array().length; i++) {
			if (fl.getCh7Gps().getCh7array()[i].getLatitude() == 0 || fl.getCh7Gps().getCh7array()[i].getLongitude() == 0) {
				System.out.println("LATITUDE " + i);
			}
		}
		// datum a cas
		// System.out.println("fl.getCh7Gps().getTime(fl.getTimestampMin()) " +
		// fl.getCh7Gps().getTime(fl.getTimestampMin()));
		// System.out.println("fl.getCh7Gps().getTime(fl.getTimestampMax()) " +
		// fl.getCh7Gps().getTime(fl.getTimestampMax()));
		// System.out.println("///////////");
		// System.out.println("fl.getCh7Gps().getCh7array()[0].getDateInt() " +
		// fl.getCh7Gps().getCh7array()[0].getDateString());
		// System.out.println("fl.getCh7Gps().getCh7array()[0].getTimeInt() " +
		// fl.getCh7Gps().getCh7array()[0].getTimeString());
		// System.out.println();
		// vypis GPS ze Zaonu
		// System.out.println("fl.getZaonSimulationData().getTimeMin() " +
		// fl.getZaonSimulationData().getTimeMin());
		// System.out.println("fl.getZaonSimulationData().getTimeMax() " +
		// fl.getZaonSimulationData().getTimeMax());
		// System.out.println("fl.getZaonSimulationData().getZaonArray()[0].getPressureAltitude() "+
		// fl.getZaonSimulationData().getZaonArray()[0].getPressureAltitudeFT()+"[ft]");
		// System.out.println();
		// altitude Zaon FT
		System.out.println("fl.getZaonSimulationData().getZaonArray()[0].getPressureAltitudeFT() "
				+ fl.getZaonSimulationData().getZaonArray()[0].getPressureAltitudeFT());

		// stredni casy
		String meanTimeIntegra = fl.getCh7Gps().getCh7array()[fl.getCh7Gps().getCh7array().length / 2].getTimeString();
		String meanTimeZaon = fl.getZaonSimulationData().getZaonArray()[fl.getZaonSimulationData().getZaonArray().length / 2]
				.getTimeString();
		// System.out.println("meanTimeZaon    =" + meanTimeZaon);
		// System.out.println("meanTimeIntegra =" + meanTimeIntegra);
		// System.out.printf("rozdil          =%06d",
		// (meanTimeZaon-meanTimeIntegra));

		Date dateZ = null;
		Date dateI = null;

		SimpleDateFormat sdf = new SimpleDateFormat("hhmmss");
		try {
			System.out.println(">>>>>>>>>>>>Rozdil<<<<<<<<<<<<");
			dateZ = sdf.parse(meanTimeZaon);// "000000");//
			dateI = sdf.parse(meanTimeIntegra); // "000100");//
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// iniciace kalendaru
		Calendar calZ = GregorianCalendar.getInstance(Locale.GERMAN);
		Calendar calI = GregorianCalendar.getInstance(Locale.GERMAN);
		Calendar calDopocitany = GregorianCalendar.getInstance(Locale.GERMAN);
		// nastaveni datumu
		calZ.setTime(dateZ);
		calI.setTime(dateI);
//		long rozdilMili = (calZ.getTimeInMillis() - calI.getTimeInMillis());
		long rozdilMili = (dateZ.getTime() - dateI.getTime());
		
		// System.out.println("rozdilMini " + rozdilMili);
		calDopocitany.setTimeInMillis(calI.getTimeInMillis() + rozdilMili);// DUlezity
																			// radek
		// kontrolni vypisy
		/*
		 * System.out.println("calZ.getTimeInMillis() " +
		 * calZ.getTimeInMillis()); System.out.println("calI.getTimeInMillis() "
		 * + calI.getTimeInMillis());
		 * System.out.println("calRozdil.getTimeInMillis() " +
		 * calRozdil.getTimeInMillis());
		 * System.out.println("calDopocitany.getTimeInMillis() " +
		 * calDopocitany.getTimeInMillis());
		 */

		System.out
				.printf("calZ %02d %02d %02d %n", calZ.get(Calendar.HOUR), calZ.get(Calendar.MINUTE), calZ.get(Calendar.SECOND));
		System.out
				.printf("calI %02d %02d %02d %n", calI.get(Calendar.HOUR), calI.get(Calendar.MINUTE), calI.get(Calendar.SECOND));
		System.out.printf("dopo %02d %02d %02d %n", calDopocitany.get(Calendar.HOUR), calDopocitany.get(Calendar.MINUTE),
				calDopocitany.get(Calendar.SECOND));
		System.out.println();
		// System.out.println("rozdil          =" +
		// sdf.format(calRozdil.getTimeInMillis()));

	}
}
