package atc.safefly.communication;

import java.io.FileNotFoundException;
import java.io.IOException;

import aglobex.simulation.entityinfo.cartesian.CartesianEntityPositionData;
import atc.ontology.gpsflightplan.GpsState;
import atc.ontology.gpsflightplan.math.GpsPosition;
import atc.safefly.externaldata.integralog.Ch5Horizon;
import atc.safefly.externaldata.integralog.Ch7Gps;
import atc.safefly.externaldata.integralog.Ch9Compass;
import atc.safefly.externaldata.integralog.NoDataInFileException;

public class Flight {

    private Ch5Horizon ch5Horizon;
    private Ch7Gps ch7Gps;
    private Ch9Compass ch9Compas;
    private ZaonSimulationData zaonSimulationData;

    private long timestampMin;
    private long timestampMax;

    public Flight(String integraFilename, String zaonFilename) {
	init(integraFilename, zaonFilename, false);
    }

    public Flight(String integraFilename, String zaonFilename,
	    boolean loadOnlyGps) {
	init(integraFilename, zaonFilename, loadOnlyGps);
    }

    private void init(String integraFilename, String zaonFilename,
	    boolean loadOnlyGps) {
	if (loadOnlyGps) {
	    try {
		ch7Gps = new Ch7Gps(integraFilename);
	    } catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	    } catch (NoDataInFileException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	    }
	    timestampMin = ch7Gps.getTimestampMin();
	    timestampMax = ch7Gps.getTimestampMax();
	} else {
	    // load channels
	    try {
		ch5Horizon = new Ch5Horizon(integraFilename);
		ch7Gps = new Ch7Gps(integraFilename);
		ch9Compas = new Ch9Compass(integraFilename);
	    } catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	    } catch (NoDataInFileException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	    }

	    // compute minimal and maximal timestamp
	    long timestampMin;
	    long timestampMax;

	    // for this flight we are concern about ch7times only
	    timestampMin = ch7Gps.getTimestampMin();
	    timestampMax = ch7Gps.getTimestampMax();
	    
	    // try ch5
//	    timestampMin = ch5Horizon.getTimestampMin();
//	    timestampMax = ch5Horizon.getTimestampMax();

	    // try ch7
//	    if (ch7Gps.getTimestampMin() < timestampMin
//		    && ch7Gps.getTimestampMin() != 0)
//		timestampMin = ch7Gps.getTimestampMin();
//	    if (ch7Gps.getTimestampMax() > timestampMax)
//		timestampMax = ch7Gps.getTimestampMax();

	    // try ch9
//	    if (ch9Compas.getTimestampMin() < timestampMin
//		    && ch9Compas.getTimestampMin() != 0)
//		timestampMin = ch9Compas.getTimestampMin();
//	    if (ch9Compas.getTimestampMax() > timestampMax)
//		timestampMax = ch9Compas.getTimestampMax();

	    // save it
	    this.timestampMin = timestampMin;
	    this.timestampMax = timestampMax;
	}

	// load Zaon simulation data
	try {
	    zaonSimulationData = new ZaonSimulationData(zaonFilename);
	} catch (FileNotFoundException e) {
	    // TODO Auto-generated catch block
	    System.err.println("file " + zaonFilename + " not found");
	    e.printStackTrace();
	} catch (NoDataInFileException e) {
	    // TODO Auto-generated catch block
	    System.err.println("no valid data in file " + zaonFilename);
	    e.printStackTrace();
	}

    }

    public long getTimestampMin() {
	return timestampMin;
    }

    public long getTimestampMax() {
	return timestampMax;
    }

    public Ch7Gps getCh7Gps() {
        return ch7Gps;
    }

    public Ch9Compass getCh9Compas() {
		return ch9Compas;
	}

	public CartesianEntityPositionData getGEPositionData(long timestamp,
	    long zeroTime, double xAdd, double yAdd, double xMul, double yMul,
	    double vPosition) {
	CartesianEntityPositionData cepd = CartesianEntityPositionData
		.getInstance(CartesianEntityPositionData.class);

	long realTime = timestamp / 1000 + zeroTime;
	System.out.println("zeroTime: " + zeroTime + ", simTime: " + timestamp
		+ ", realTime: " + realTime);
	// extract data
	Ch5Horizon.Ch5Row ch5 = ch5Horizon.getCh5Row(realTime);
	Ch7Gps.Ch7Row ch7 = ch7Gps.getCh7Row(realTime);
	Ch9Compass.Ch9Row ch9 = ch9Compas.getCh9Row(realTime);

	System.out.println(ch7.longitude + " " + ch7.latitude);

	// set values
	cepd.x = (ch7.longitude - xAdd) * xMul;
	cepd.y = (ch7.latitude - yAdd) * yMul;
	cepd.z = ch7.altitude - vPosition;
	cepd.roll = -1 * ch5.roll / 180 * Math.PI;
	cepd.pitch = -1 * ch5.tilt / 180 * Math.PI;
	cepd.yaw = /* Math.PI/2 - */Math.PI - ch9.heading / 180 * Math.PI;

	// System.out.println(cepd);

	return cepd;
    }

    public GpsPosition getGpsPosition(long timestamp) {
	Ch7Gps.Ch7Row ch7 = ch7Gps.getCh7Row(timestamp);

	GpsPosition pos = new GpsPosition();
	pos.setGpsPosition(ch7.longitude, ch7.latitude, ch7.altitude);

	return pos;
    }

    public GpsState getGpsState(long timestamp) {
	Ch5Horizon.Ch5Row ch5 = ch5Horizon.getCh5Row(timestamp);
	Ch7Gps.Ch7Row ch7 = ch7Gps.getCh7Row(timestamp);
	Ch9Compass.Ch9Row ch9 = ch9Compas.getCh9Row(timestamp);

	double roll = -1 * ch5.roll / 180 * Math.PI;
	double pitch = -1 * ch5.tilt / 180 * Math.PI;
	double heading = (ch9.heading + 270) % 360;

	GpsState state = new GpsState();
	state.setState(timestamp, ch7.longitude, ch7.latitude, ch7.altitude,
		heading, pitch, roll, 0, 0, 0, 0);

	return state;
    }

    public ZaonSimulationData getZaonSimulationData() {
        return zaonSimulationData;
    }
    
    
    

}
