package atc.safefly.communication;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.util.Arrays;
import java.util.LinkedList;

import atc.procerus.communication.ConnectionOwner;
import atc.procerus.communication.ProcerusPacket;
import atc.procerus.communication.UAVConnector;
import atc.safefly.communication.common.serial.SerialConnector;
import atc.safefly.communication.utils.ZaonPacket;

public class ZaonConnector implements UAVConnector {

	protected ConnectionOwner owner;
	protected SerialConnector slaveConnector;

	protected InputStream inputStream;
	protected OutputStream outputStream;
	protected boolean isConnected = false;
	protected ReaderThread reader;

	/** List of packets that have not been read by the application */
	public LinkedList<ProcerusPacket> packetContainer = new LinkedList<ProcerusPacket>();
	/** Object used to synchronizing the access to the {@link #packetContainer} */
	protected Object lock = new Object();

	public ZaonConnector(ConnectionOwner owner, SerialConnector connector) {
		this.owner = owner;
		this.slaveConnector = connector;
	}

	@Override
	public boolean connect() throws IOException {
		// serialPort = connectToPort(portName, baudRate);
		if (slaveConnector.connect()) {
			outputStream = slaveConnector.getOutputStream();
			inputStream = slaveConnector.getInputStream();
			isConnected = true;
			reader = new ReaderThread(owner, this);
			reader.start();
			return true;
		}

		// register listener
		// try {
		// // readData();
		// isConnected = true;
		// // serialPort.addEventListener(this);
		// return true;
		// } catch (TooManyListenersException e) {
		// e.printStackTrace();
		// }
		return false;
	}

	@Override
	public boolean sendPacket(ProcerusPacket pPacket) {
		if (isConnected) {
			// if (pPacket instanceof SafeflyPacket) {
			// SafeflyPacket packet = (SafeflyPacket) pPacket;
			// byte[] bytesToSend = packet.assemblyPacket();
			// try {
			// outputStream.write(bytesToSend);
			// outputStream.flush();
			// // debug print
			// System.out.println("Sent: "
			// + ByteUtils.byteArrayToHexString(bytesToSend));
			// return true;
			// } catch (IOException e) {
			// isConnected = false;
			// e.printStackTrace();
			// }
			// }
		}
		return false;
	}

	@Override
	public ProcerusPacket pollNextPacket() {
		synchronized (lock) {
			return packetContainer.pollFirst();
		}
	}

	@Override
	public void dispose() {
		try {
			if (inputStream != null) {
				inputStream.close();
			}
			if (outputStream != null) {
				outputStream.close();
			}
			if (slaveConnector != null) {
				slaveConnector.dispose();
			}
			if (reader != null) {
				reader.deactivate();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private class ReaderThread extends Thread {

		protected ConnectionOwner owner;
		protected UAVConnector connector;
		protected boolean active;

		public ReaderThread(ConnectionOwner owner, UAVConnector connector) {
			this.owner = owner;
			this.connector = connector;
			this.active = true;
		}

		@Override
		public void run() {

			while (active) {
				readData();
			}
		}

		public void deactivate() {
			active = false;
		}

		protected void readData() {
			int val;
			byte bVal;
			// int recievedDataLength = 0;
			try {
				if ((val = inputStream.read()) != -1) {
					bVal = (byte) val;
					ByteBuffer bb = ByteBuffer.allocate(1024);
					bb.put(bVal);

					for (int i = 0; i < 21; i++) {
						val = inputStream.read();
						bVal = (byte) val;
						bb.put(bVal);
						if (val == -1) {
							System.err.println("ZaonConnector: end of stream !");
							return;
						}
					}

					ZaonPacket zPacket = new ZaonPacket();

					if (bb.hasArray()) {
						if (zPacket.decodePacketBytes(Arrays.copyOfRange(bb.array(), 0, 22))) {
							synchronized (lock) {
								packetContainer.addLast(zPacket);
							}
							// notify the owner, that we received new data
							owner.incommingMessage(connector);

						} else {
							System.out.println("Checksum failed");
							inputStream.skip(inputStream.available());
						}
					} else {
						System.out.println("ByteBuffer is empty");
					}

				} else {
					isConnected = false;
					System.out.println("Connection failed");
				}
			} catch (IOException e) {
				isConnected = false;
				System.out.println("Not connected ...");// TODO take care of
				// this
				e.printStackTrace();
			}
		}

	}

}
