package atc.safefly.communication;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import atc.safefly.communication.utils.ZaonPacket;
import atc.safefly.externaldata.integralog.NoDataInFileException;
import au.com.bytecode.opencsv.CSVReader;

public class ZaonSimulationData {

    private ZaonRow[] zaonArray;

    private String timeMin;
    private String timeMax;

    public ZaonSimulationData(String zaonFilename)
	    throws FileNotFoundException, NoDataInFileException {

	// initialize CSV reader
	CSVReader reader = new CSVReader(new FileReader(zaonFilename), ' ');
	String[] line;
	ArrayList<ZaonRow> recordList = new ArrayList<ZaonRow>();

	// read lines of file
	try {
	    while ((line = reader.readNext()) != null) {
		if (!line[0].contains(".")) {
		    continue;
		}
		// creating date String for comparsion with Integra date
		// "ddMMyy"
		String dateString = "";
		dateString += String.format(
			"%02d",
			Integer.parseInt(line[0].substring(0,
				line[0].indexOf(".", 0))));
		dateString += String.format(
			"%02d",
			Integer.parseInt(line[0].substring(
				line[0].indexOf(".", 0) + 1,
				line[0].lastIndexOf("."))));
		dateString += line[0].substring(line[0].length() - 2,
			line[0].length());

		// creation of time String for comparsion with Integra time
		// "hhmmss"
		String timeString = line[1].replace(":", "");

		// parsing packet bytes
		byte[] packetBytes = new byte[22];
		for (int i = 3; i < line.length - 1; i++) {
		    try {
			int ipom = Integer.parseInt(line[i], 16);
			packetBytes[i - 3] = (byte) ipom;
		    } catch (Exception e) {
			System.out.println("> skipping entry " + i + ">"
				+ Byte.parseByte(line[i], 16));
			break;
		    }
		}

		// create and fill ZaonRow
		ZaonRow row = new ZaonRow(dateString, timeString, packetBytes);
		recordList.add(row);
	    }

	} catch (NumberFormatException | IOException e) {
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	}
	try {
	    reader.close();
	} catch (IOException e) {
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	}
	if (recordList.isEmpty())
	    throw new NoDataInFileException(zaonFilename, 7);

	// save array
	zaonArray = new ZaonRow[1];
	zaonArray = recordList.toArray(zaonArray);

	// assign min and max time
	timeMin = (zaonArray[0].getTimeString());
	timeMax = (zaonArray[zaonArray.length - 1].getTimeString());
    }

    public String getTimeMin() {
	return timeMin;
    }

    public String getTimeMax() {
	return timeMax;
    }
    
    

    public ZaonRow[] getZaonArray() {
        return zaonArray;
    }



    // represents one row in Zaon log file
    public class ZaonRow extends ZaonPacket {
	private String dateString;
	private String timeString;

	public ZaonRow(String dateString, String timeString, byte[] packetBytes) {
	    super(packetBytes);
	    this.dateString = dateString;
	    this.timeString = timeString;
	}

	public String getDateString() {
	    return dateString;
	}

	public void setDateString(String dateString) {
	    this.dateString = dateString;
	}

	public String getTimeString() {
	    return timeString;
	}

	public void setTimeString(String timeString) {
	    this.timeString = timeString;
	}

    }

}
