package atc.safefly.communication;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.util.Date;
import java.util.Locale;

import atc.procerus.communication.ConnectionOwner;
import atc.procerus.communication.ProcerusPacket;
import atc.procerus.communication.ServerSerialConnector;
import atc.procerus.communication.SocketSerialConnector;
import atc.procerus.communication.UAVConnector;
import atc.procerus.utils.ByteUtils;
import atc.safefly.communication.common.serial.UniversalSerialConnector;
import atc.safefly.communication.utils.ZaonPacket;
import atc.safefly.ontology.SerialPortSettings;
import atc.safefly.utils.XmlLoader;
import java.util.Scanner;

public class ZaonTestApp implements ConnectionOwner {

	UAVConnector connector = null;

	private String port = "COM3"; // default port
	private int baudRate = 57600; // default baudRate

	protected BufferedWriter bwLogFile;
	protected BufferedWriter bwLogFileRaw;

	public String getPort() {
		return port;
	}

	public void setPort(String port) {
		this.port = port;
	}

	public int getBaudRate() {
		return baudRate;
	}

	public void setBaudRate(int baudRate) {
		this.baudRate = baudRate;
	}

	protected void initiateLogFiles() {
		// creating logs directory
		File dir = new File("zaon_test_app_log_files");
		if (!dir.exists()) {
			System.out.println("creating logs directory: "
					+ dir.getAbsolutePath());
			boolean result = dir.mkdir();
			if (result) {
				System.out.println("Directory created");
			}
			System.out.println();
		}
		// creating logfiles
		try {
			int i = 1;
			File file = new File(dir + "/zaon_log(" + i + ").txt");
			File rFile = new File(dir + "/zaon_log(" + i + ")raw.txt");
			while (file.exists() || rFile.exists()) {
				i++;
				file = new File(dir + "/zaon_log(" + i + ").txt");
				rFile = new File(dir + "/zaon_log(" + i + ")raw_data.txt");
			}
			bwLogFile = new BufferedWriter(new FileWriter(file));
			bwLogFileRaw = new BufferedWriter(new FileWriter(rFile));

			System.out.printf(
					"Created log file     > %s%nCreated raw lof file > %s%n",
					file, rFile);
		} catch (IOException ex1) {
			System.err.println("ZaonTestApp:Cannot create log file: " + ex1);
		}
	}

	public ZaonTestApp() {
	}

	public ZaonTestApp(String port, int baudRate) {

		initiateLogFiles();

		// diferrent connector for linux and windows env.
		connector = new ZaonConnector(this, new UniversalSerialConnector(port,
				baudRate));
		try {
			connector.connect();
		} catch (IOException e) {
			e.printStackTrace();
			this.log("Unable connect connector: " + e);
		}
	}

	public static String xorCheckToIntString(byte[] pole) {
		String message;

		// creation of log message
		message = String.format("");

		for (byte b : pole) {
			// System.out.print((b ^ 0x76)+" ");
			b = (byte) (b & 0xFF);
			if (b == 0x00 || b == 0x76) {
				message += String.format("%s ", b);
			} else {
				message += String.format("%s ", (b ^ 0x76));
			}
		}
		message += String.format("%n");
		return message;
	}

	public static String decryptedToHexString(byte[] pole) {
		String message;

		// creation of log message
		message = String.format("");

		for (byte b : pole) {
			// System.out.print((b ^ 0x76)+" ");
			b = (byte) (b & 0xFF);
			if (b == 0x00 || b == 0x76) {
				message += String.format("%s ", b);
			} else {
				message += String.format("%s ", (b ^ 0x76));
			}
		}
		message += String.format("%n");
		return message;
	}

	@Override
	public void incommingMessage(UAVConnector connector) {
		ProcerusPacket packet;
		while ((packet = connector.pollNextPacket()) != null) {
			processPacket(packet);
		}
	}

	public void log(String log) {
		if (bwLogFile != null) {
			try {
				String time = DateFormat.getDateTimeInstance(DateFormat.MEDIUM,
						DateFormat.MEDIUM, Locale.GERMAN).format(new Date());
				bwLogFile.append(time + " : ");
				bwLogFile.newLine();
				bwLogFile.append(log);
				bwLogFile.newLine();
				bwLogFile.flush();
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}
	}

	public void lograw(String logRaw) {
		if (bwLogFileRaw != null) {
			try {

				String time = DateFormat.getDateTimeInstance(DateFormat.MEDIUM,
						DateFormat.MEDIUM, Locale.GERMAN).format(new Date());
				bwLogFileRaw.append(time + " : " + logRaw);
				bwLogFileRaw.newLine();
				bwLogFileRaw.flush();
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}
	}

	synchronized private void processPacket(ProcerusPacket packet) {
		String message = "";
		String messageRaw = "";

		// creation of log message
		// message = String.format("Recieved new ZaonPacket :%n");

		if (packet instanceof ZaonPacket) {
			ZaonPacket zPacket = (ZaonPacket) packet;
			message += zPacket.toString(true, true, true, true);
			messageRaw += zPacket.toString(true, false, false, false);
		} else {
			message += "recieved packet is not instance of ZaonPacket >";
			message += String.format("%s%n",
					ByteUtils.byteArrayToHexString(packet.getPacketAsBytes()));
			messageRaw = message;
		}

		System.out.println(message);
		this.log(message);
		this.lograw(messageRaw);
	}

	// prints out help text for main() nethod
	private static void printHelp() {
		System.out.println();
		System.out
				.println("============================< ZaonTestApp Client Help >============================");
		System.out.println();
		System.out.println("NAME");
		System.out
				.println("    ZaonTestApp.jar --Zaon client for Zaon-Gumstix communication");
		System.out.println();
		System.out.println("ARGUMENTS");
		// System.out.println();
		System.out.println("    [no arguments | -h | -help]");
		System.out.println("                --print out this help");
		System.out.println();
		System.out.println("    [-d | -default]  ");
		System.out
				.println("                --load configuration from default location");
		System.out.println("                  xml/zaon_serial_port.xml");
		System.out.println();
		System.out
				.println("    [-portxmlfile | -portfile | -pf ] filepath    ");
		System.out
				.println("                --set path to Port Xml configuration file");
		System.out.println();
		System.out.println("DESCRIPTION");
		// System.out.println();
		System.out.println("    Parse and logs incoming datastream.");
		System.out
				.println("    Port configuration is read from external xml files.");
		System.exit(-1);
	}

	private static String nextArg(int i, String[] args) {
		if (i + 1 == args.length) {
			System.err.println("missing parameter after marker " + args[i]);
			System.exit(-1);
		}
		return args[i + 1];
	}

	public static String[] parseInputArguments(String[] arguments) {
		// defining default values
		final String DEFAULT_PORT_XML_LOCATION = "xml/zaon_serial_port.xml";

		String portXmlLocation = "";

		System.out.print("running with imput arguments of: ");
		for (String string : arguments) {
			System.out.print(string);
		}
		System.out.println();

		// parsing input agruments
		if (arguments.length == 0) {
			printHelp();
		}
		for (int i = 0; i < arguments.length; i++) {
			if (arguments[i].equals("-h") | arguments[i].equals("-help")) {
				printHelp();
			}
			if (arguments[i].equals("-d") | arguments[i].equals("-default")) {
				portXmlLocation = DEFAULT_PORT_XML_LOCATION;
				break;
			}
			if (arguments[i].equals("-portxmlfile")
					| arguments[i].equals("-portfile")
					| arguments[i].equals("-pf")) {
				portXmlLocation = nextArg(i++, arguments);
				continue;
			}

		}
		String[] result = { portXmlLocation };
		return result;
	}

	public static void main(String[] args) {

		String lMessage = "";
		String portXmlLocation = "";

		lMessage = String.format("ZaonClient running...%n%n");
		System.out.printf(lMessage);

		String port;// = "COM05";
		int baudRate;// = 57600;

		String[] parsingResult = parseInputArguments(args);
		portXmlLocation = parsingResult[0];

		SerialPortSettings com = (SerialPortSettings) XmlLoader.loadXml(
				portXmlLocation, SerialPortSettings.class);

		port = com.getPort();
		baudRate = com.getBaudRate();

		System.out.println("port > " + port);
		System.out.println("baudRate > " + baudRate);
		ZaonTestApp gClient = new ZaonTestApp(port, baudRate);

		gClient.log(lMessage);

		Scanner sc = new Scanner(System.in);

		System.out.println("Return to send and Q to quit");
		while (!sc.nextLine().equalsIgnoreCase("q")) {
		}
		sc.close();
		gClient.connector.dispose();

	}
}
