package atc.safefly.communication.common.packet;

import java.nio.ByteBuffer;

public class AddressedPacket extends Packet {

	private static final int HEADER_SIZE = 4;
	/*************** Packet Factory registration ************/
	public static final short ID = 0x0100;
	static {
		PacketFactory.registerPacket(new AddressedPacket());
	}

	public AddressedPacket() {
	}

	@Override
	public Packet getInstance() {
		return new AddressedPacket();
	}

	@Override
	public short getApplicationID() {
		return ID;
	}

	/*********************************************************/

	public static final int BROADCAST = 0xFFFF;

	protected short srcAddr;
	protected short dstAddr;

	public AddressedPacket(int destinationAddress, int sourceAddress, byte[] data) {
		super(data);
		this.srcAddr = (short) sourceAddress;
		this.dstAddr = (short) destinationAddress;
	}

	public AddressedPacket(int sourceAddress, byte[] data) {
		this(BROADCAST, sourceAddress, data);
	}

	public AddressedPacket(ByteBuffer bb, int length) {
		deserialize(bb, length);
	}

	@Override
	public byte[] serialize() {
		ByteBuffer bb = ByteBuffer.allocate(HEADER_SIZE + data.length);
		bb.putShort(dstAddr);
		bb.putShort(srcAddr);
		bb.put(data);

		return bb.array();
	}

	@Override
	public boolean deserialize(ByteBuffer bb, int length) {
		if(length < HEADER_SIZE)
			return false;
		dstAddr = bb.getShort();
		srcAddr = bb.getShort();
		data = new byte[length - HEADER_SIZE];
		bb.get(data);
		return true;
	}

	@Override
	public int getSize() {
		return super.getSize() + HEADER_SIZE;
	}

	@Override
	public int getHeaderSize() {
		return super.getHeaderSize() + HEADER_SIZE;
	}

	@Override
	public String toString() {
		return String.format("Addressed: [%d > %d] {%s}", srcAddr & 0xFFFF, dstAddr & 0xFFFF, super.toString());
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result + dstAddr;
		result = prime * result + srcAddr;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		AddressedPacket other = (AddressedPacket) obj;
		if (dstAddr != other.dstAddr)
			return false;
		if (srcAddr != other.srcAddr)
			return false;
		return true;
	}

	public short getSrcAddr() {
		return srcAddr;
	}

	public short getDstAddr() {
		return dstAddr;
	}
	
	
}
