package atc.safefly.communication.common.packet;

import java.nio.ByteBuffer;
import java.util.Arrays;

import atc.safefly.utils.ByteUtils;

public class Packet implements PacketInterface {
	/*************** Packet Factory registration ************/
	public static final short ID = 0x0000;
	static {
		PacketFactory.registerPacket(new Packet());
	}

	@Override
	public Packet getInstance() {
		return new Packet();
	}

	@Override
	public short getApplicationID() {
		return ID;
	}

	/*********************************************************/

	protected byte[] data;

	public Packet() {
	}

	public Packet(byte[] data) {
		this.data = data;
	}

	public Packet(ByteBuffer bb, int length) {
		deserialize(bb, length);
	}

	@Override
	public byte[] getData() {
		return data;
	}
	
	public void setData(byte[] data) {
		this.data = data;
	}

	@Override
	public boolean deserialize(ByteBuffer bb, int length) {
		data = new byte[length];
		bb.get(data);
		return true;
	}

	@Override
	public byte[] serialize() {
		byte[] data = new byte[this.data.length];
		System.arraycopy(this.data, 0, data, 0, data.length);
		return data;
	}

	@Override
	public int getSize() {
		if (data != null) {
			return data.length;
		}
		return 0;
	}

	@Override
	public int getHeaderSize() {
		return 0;
	}

	@Override
	public String toString() {
		if (data != null) {
			return String.format("Packet: [ID: 0x%s] [Data: %s]", Integer.toHexString(getApplicationID()), ByteUtils.byteArrayToHexString(data));
		} else {
			return String.format("Packet: [ID: 0x%s] [Data: XXX]", Integer.toHexString(getApplicationID()));
		}
	}

	@Override
	public int hashCode() { 
		return Arrays.hashCode(data);
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Packet other = (Packet) obj;
		if (!Arrays.equals(data, other.data))
			return false;
		return true;
	}
	
	

}
