package atc.safefly.communication.common.serial;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

public abstract class AbstractSerialConnector implements SerialConnector {
	protected InputStream in;
	protected boolean inLock;
	protected OutputStream out;
	protected boolean outLock;

	protected boolean isConnected;

	public AbstractSerialConnector() {
		in = null;
		inLock = false;
		out = null;
		outLock = false;
		isConnected = false;
	}
	
	@Override
	public boolean isConnected() {
		if (isConnected) {
			// Check stream connection
			try {
				if (in != null) {
					in.available();
				} else {
					isConnected = false;
				}
			} catch (IOException e) {
				dispose();
			}
		}
		return isConnected;
	}

	@Override
	public InputStream getInputStream() {
		if(isConnected && !inLock){
			inLock = true;
			return in;
		}
		return null;
	}

	@Override
	public OutputStream getOutputStream() {
		if(isConnected && !outLock){
			outLock = true;
			return out;
		}
		return null;
	}

	@Override
	public void restoreStreams() {
		inLock = false;
		outLock = false;
	}

}
