/**
 * 
 */
package atc.safefly.communication.common.serial;

import gnu.io.CommPortIdentifier;
import gnu.io.PortInUseException;
import gnu.io.SerialPort;
import gnu.io.SerialPortEvent;
import gnu.io.SerialPortEventListener;
import gnu.io.UnsupportedCommOperationException;

import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.TooManyListenersException;


/**
 * @author Steefa
 * 
 */
public class RxtxSerialConnector extends AbstractSerialConnector implements SerialPortEventListener {

	private InputStream rxtxIn;
	private Object inputReadyLock = new Object();

	private String portName;
	private SerialPort serialPort;
	private int baudrate;

	public RxtxSerialConnector(String portName, int baudrate) {
		super();
		this.portName = portName;
		this.baudrate = baudrate;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see atc.procerus.communication.SerialConnector#connect()
	 */
	@Override
	public boolean connect() {
		boolean portFound = false;
		@SuppressWarnings("unchecked")
		Enumeration<CommPortIdentifier> portList = CommPortIdentifier.getPortIdentifiers();
		CommPortIdentifier portId = null;
		while (portList.hasMoreElements()) {
			portId = portList.nextElement();
			if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
				if (portId.getName().equals(portName)) {
					System.out.println("Found port: " + portName);
					portFound = true;
					break;
				}
			}
		}

		if (!portFound) {
			System.out.println("Port " + portName + " not found.");
			return false;
		}

		try {
			serialPort = (SerialPort) portId.open("Serial Connector", 2000);
		} catch (PortInUseException e1) {
			e1.printStackTrace();
			return false;
		}
		try {
			// set port parameters
			serialPort.setSerialPortParams(baudrate, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
			// serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_XONXOFF_OUT
			// | SerialPort.FLOWCONTROL_XONXOFF_IN);
		} catch (UnsupportedCommOperationException e) {
			e.printStackTrace();
		}
		serialPort.notifyOnDataAvailable(true);
		try {
			serialPort.addEventListener(this);
		} catch (TooManyListenersException e1) {
			e1.printStackTrace();
			return false;
		}
		try {
			out = serialPort.getOutputStream();
			rxtxIn = serialPort.getInputStream();
			// Blocking synchronized read provider
			in = new InputStream() {

				@Override
				public int read() throws IOException {
					synchronized (inputReadyLock) {
						int val = rxtxIn.read();
						if (val == -1) {
							try {
								inputReadyLock.wait();
							} catch (InterruptedException e) {
								return -1;
							}
						} else {
							return val;
						}
						return rxtxIn.read();
					}
				}
			};
			isConnected = true;
			return true;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see atc.procerus.communication.SerialConnector#dispose()
	 */
	@Override
	public void dispose() {
		try {
			if (in != null) {
				in.close();
			}
			if (out != null) {
				out.flush();
				out.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		if (serialPort != null) {
			serialPort.removeEventListener();
			serialPort.close();
		}
	}

	@Override
	public void serialEvent(SerialPortEvent event) {
		synchronized (inputReadyLock) {
			switch (event.getEventType()) {
			// case SerialPortEvent.BI:
			// case SerialPortEvent.OE:
			// case SerialPortEvent.FE:
			// case SerialPortEvent.PE:
			// case SerialPortEvent.CD:
			// case SerialPortEvent.CTS:
			// case SerialPortEvent.DSR:
			// case SerialPortEvent.RI:
			// case SerialPortEvent.OUTPUT_BUFFER_EMPTY:
			case SerialPortEvent.DATA_AVAILABLE:
				inputReadyLock.notifyAll();
				break;
			}
		}
	}

	@Override
	public String getPortName() {
		return portName;
	}

}
