package atc.safefly.communication.common.serial;

import java.io.InputStream;
import java.io.OutputStream;

public interface SerialConnector {

	/**
	 * Check if connector is connected
	 * @return true if connected, false otherwise
	 */
	public boolean isConnected();

	/**
	 * Connect to underlying socket and starts maintenance process
	 * @return true if connection succeed, false otherwise
	 */
	public boolean connect();

	/**
	 * Close sockets and destroy all underlying structures
	 */
	public void dispose();

	/**
	 * Give input stream for reading from serial
	 * @return InputStream of underlying socket, NULL if connector is closed or another connector is using this stream
	 */
	public InputStream getInputStream();

	/**
	 * Give output stream for writing to serial
	 * @return OutputStream of underlying socket, NULL if connector is closed or another connector is using this stream
	 */
	public OutputStream getOutputStream();

	/**
	 * Indicates upper connector stop using each Streams 
	 */
	public void restoreStreams();
	
	public String getPortName();

}
