package atc.safefly.communication.modem;

import atc.safefly.communication.common.packet.PacketConnectionOwner;
import atc.safefly.communication.common.serial.SerialConnector;

public class AddressedDataPacketConnector extends DataPacketConnector {

	int address;

	public AddressedDataPacketConnector(SerialConnector subConnector, PacketConnectionOwner owner, int address) {
		super(subConnector, owner);
		this.address = address;
	}

	public AddressedDataPacketConnector(SerialConnector subConnector, int address) {
		super(subConnector);
		this.address = address;
	}

	public AddressedDataPacketConnector(SerialConnector subConnector, PacketConnectionOwner owner) {
		super(subConnector, owner);
	}

	public AddressedDataPacketConnector(SerialConnector subConnector) {
		super(subConnector);
	}

	@Override
	protected boolean open() {
		return setAddress() && super.open();
	}

	/**
	 * Method open Terminal connection to modem and use AT commands to set or
	 * read modem unit address
	 * 
	 * @return true if succeed, false otherwise
	 */
	private boolean setAddress() {
		TerminalConnector tc = new TerminalConnector(subConnector);
		if (tc.connect()) {
			if (address > 0) {
				tc.setAddress(address);
			} else {
				address = tc.getAddress();
			}
			System.out.println("Current unit address is " + address);
			tc.close();
			return true;
		} else {
			System.err.println("Failed to set modem unit address=" + address);
			return false;
		}
	}

	public int getAddress() {
		return address;
	}

}
