package atc.safefly.communication.test;

import java.util.HashMap;
import java.util.Random;
import java.util.Scanner;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.ReentrantLock;

import atc.safefly.communication.common.SafeflyConnector;
import atc.safefly.communication.common.packet.FragmentPacket;
import atc.safefly.communication.common.packet.MessageConnectionOwner;
import atc.safefly.communication.common.packet.MessageConnector;
import atc.safefly.communication.common.packet.Packet;
import atc.safefly.communication.common.packet.PacketConnectionOwner;
import atc.safefly.communication.common.packet.PacketInterface;
import atc.safefly.communication.common.serial.UniversalSerialConnector;
import atc.safefly.communication.utils.MessageRecord;
import atc.safefly.communication.utils.MessageRecordHandler;

public class MessageConnectorTest {

	private static final boolean verbose = false;
	private static final Random r = new Random();

	private static final int MAX_FAILURE_DELAY = 30000; // ms
	private static final int MIN_FAILURE_DELAY = 10000; // ms

	private static final int MAX_FAILURE_DURATION = 10000;
	private static final int MIN_FAILURE_DURATION = 4000;
	private static final Timer failTimer;
	private static TimerTask failTask;

	private static final short adr1 = 1;
	private static final String com1 = "COM4";
	private static final short adr2 = 2;
	private static final String com2 = "COM8";
	private static final String USAGE = "Type'Q' to quit.\n" + "Message: M /bytes/\n" + "Packet: P /bytes/\n"
			+ "Auto: A /period_min/ /period_max/ /size_min/ /size_max/ /mode/\n" + "\t\t Mode:[0-Single],[1-HalfDuplex],[2-FullDuplex],[3-FullAsyncDuplex]\n"
			+ "Cance auto mode: C";

	private static HashMap<MessageRecord, Long> records = new HashMap<MessageRecord, Long>();
	private static volatile int correct;
	private static volatile long bytes;
	private static volatile long time;

	private static final SafeflyConnector line1;
	private static MessageConnector mc1;

	private static final SafeflyConnector line2;
	private static MessageConnector mc2;

	private static volatile boolean connectionFail[] = new boolean[2];
	static {
		// line1 = new SafeflyConnector(new ServerSocketConnector(5555),
		// verbose);
		failTimer = new Timer();

		line1 = new SafeflyConnector(new UniversalSerialConnector(com1, 230400), verbose);
		mc1 = new MessageConnector(line1, new MessageConnectionOwner() {

			@Override
			public void incommingMessage(MessageRecord mr) {
				if (records.containsKey(mr)) {
					long t = System.currentTimeMillis() - records.get(mr);
					long b = mr.getPacket().getSize();
					records.remove(mr);
					System.out.println(String.format("CORRECT: %dB, %dms, %.2fkB/s", b, t, (b / 1024f) / (t / 1000f)));
					correct++;
					bytes += b;
					time += t;
				} else {
					System.out.println("FAILED");
				}
			}
		}, adr1, adr2);
		PacketConnectionOwner owner1 = new PacketConnectionOwner() {

			@Override
			public void incommingPacket(PacketInterface packet) {
				// Connection fail simulation
				if (connectionFail[0])
					return;
				if (packet instanceof FragmentPacket) {
					mc1.fragmentReceived((FragmentPacket) packet);
				} else {
					System.out.println("Line1 received packet with length: " + packet.getSize());
				}

			}
		};
		line1.setOwner(owner1);

		// new Thread(new Runnable() {
		//
		// @Override
		// public void run() {
		// mc1.connect();
		// }
		// }).start();
		//
		// line2 = new SafeflyConnector(new SocketSerialConnector(5555),
		// verbose);
		line2 = new SafeflyConnector(new UniversalSerialConnector(com2, 230400), verbose);
		mc2 = new MessageConnector(line2, new MessageConnectionOwner() {

			@Override
			public void incommingMessage(MessageRecord mr) {
				if (records.containsKey(mr)) {
					long t = System.currentTimeMillis() - records.get(mr);
					long b = mr.getPacket().getSize();
					records.remove(mr);
					System.out.println(String.format("CORRECT: %dB, %dms, %.2fkB/s", b, t, (b / 1024f) / (t / 1000f)));
					correct++;
					bytes += b;
					time += t;
				} else {
					System.out.println("FAILED");
				}
			}
		}, adr2, adr1);
		PacketConnectionOwner owner2 = new PacketConnectionOwner() {

			@Override
			public void incommingPacket(PacketInterface packet) {
				// Connection fail simulation
				if (connectionFail[1])
					return;
				if (packet instanceof FragmentPacket) {
					mc2.fragmentReceived((FragmentPacket) packet);
				} else {
					System.out.println("Line2 received packet with length: " + packet.getSize());
				}

			}
		};
		line2.setOwner(owner2);

	}

	public static void main(String[] args) {
		Thread auto = null;
		Scanner sc = new Scanner(System.in);
		String line;
		System.out.println(USAGE);
		while (!(line = sc.nextLine()).equalsIgnoreCase("Q")) {
			String[] ans = line.split("\\s+");
			if (ans.length > 0) {
				int bc = 0;
				if (ans[0].equalsIgnoreCase("M") && ans.length > 1) {
					try {
						bc = Integer.parseInt(ans[1]);
						if (bc < 1) {
							System.out.println("Not valid number of bytes: " + ans[1] + "\n" + USAGE);
							continue;
						}
					} catch (NumberFormatException e) {
						System.out.println("Not valid number of bytes: " + ans[1] + "\n" + USAGE);
						continue;
					}
					byte[] data = new byte[bc];
					r.nextBytes(data);
					Packet p = new Packet(data);
					MessageRecord mr = new MessageRecord(p, adr2, adr1, ((int) Math.floor(((double) data.length) / 1.024)));
					mr.addHandler(new MessageRecordHandler() {

						@Override
						public void handleMessageRecord(MessageRecord mr, MessageRecord.State state) {
							if (state == MessageRecord.State.SENT) {
								System.out.println("Sent in " + (System.currentTimeMillis() - mr.getEnqueueTime() + " Size:" + mr.getPacket().getData().length));
							}
							System.out.println("State changed: " + mr.getState());
						}
					});

					mc1.connect();
					mc2.connect();
					records.put(mr, System.currentTimeMillis());
					mc1.sendMessage(mr);
					try {
						Thread.sleep(mr.getTimeout());
					} catch (InterruptedException e) {
						continue;
					}
					mc1.close();
					mc2.close();
					continue;
				} else if (ans[0].equalsIgnoreCase("P")) {
					int repeat = 0;
					try {
						bc = Integer.parseInt(ans[1]);
						if (bc < 1) {
							System.out.println("Not valid number of bytes: " + ans[1] + "\n" + USAGE);
							continue;
						}
						if (ans.length > 2) {
							repeat = Integer.parseInt(ans[2]);
						}
						byte[] data = new byte[bc];
						r.nextBytes(data);
						final Packet p = new Packet(data);
						final ReentrantLock lock = new ReentrantLock();
						final Condition cond = lock.newCondition();
						line2.setOwner(new PacketConnectionOwner() {

							@Override
							public void incommingPacket(PacketInterface packet) {
								lock.lock();
								try {
									cond.signal();
								}finally {
									lock.unlock();
								}
							}
						});
						line1.connect();
						line2.connect();
						final int repeatF = repeat;
						Thread sending = new Thread(new Runnable() {

							@Override
							public void run() {
								int repeat = repeatF;
								long nanos = TimeUnit.SECONDS.toNanos(1);
								lock.lock();
								try {
									do {
										repeat--;
										line1.sendPacket(p);
										long rest = cond.awaitNanos(nanos);
										System.out.println(TimeUnit.NANOSECONDS.toMillis(nanos-rest));
									} while (repeat > 0);
								} catch (InterruptedException e) {
									return;
								}finally {
									lock.unlock();
								}
							}
						});
						sending.start();
						sending.join();
					} catch (NumberFormatException e) {
						System.out.println("Not valid number of bytes: " + ans[1] + "\n" + USAGE);
						continue;
					} catch (InterruptedException e) {
						break;
					}
					continue;
				} else if (ans[0].equalsIgnoreCase("A")) {
					// AUtomatic mode
					if (ans.length < 6) {
						System.out.println("Not enough arguments: " + line + "\n" + USAGE);
						continue;
					}
					final int[] aArg = new int[5];
					int i = 0;
					try {
						for (; i < aArg.length; i++) {
							aArg[i] = Integer.parseInt(ans[i + 1]);
						}
					} catch (NumberFormatException e) {
						System.out.println("Not parseble argument(" + ans[i + 1] + "): " + line + "\n" + USAGE);
						continue;
					}
//					commFailure();
					auto = new Thread(new Runnable() {

						@Override
						public void run() {
							autoSending(aArg);
						}
					}, "Auto sending");
					System.out.println("Autosending started!");
					auto.start();

				} else if (ans[0].equalsIgnoreCase("C")) {
					if (auto != null) {
						auto.interrupt();
					}
					if (failTask != null) {
						failTask.cancel();
					}
					System.out.println("Auto mode interrupted!");
					continue;
				}else {
					System.out.println("Not valid option: " + line + "\n" + USAGE);
				}
			} else {
				System.out.println("Not valid option: " + line + "\n" + USAGE);
			}
		}
		if (auto != null) {
			auto.interrupt();
		}
		failTimer.cancel();
		sc.close();
		mc1.dispose();
		mc2.dispose();
	}

	protected static void commFailure() {
		int delay = r.nextInt(MAX_FAILURE_DELAY - MIN_FAILURE_DELAY) + MIN_FAILURE_DELAY;
		int duration = r.nextInt(MAX_FAILURE_DURATION - MIN_FAILURE_DURATION) + MIN_FAILURE_DURATION;
		final int failType = r.nextInt(4);
		int[] array = new int[0];

		switch (failType) {
			case 0:
				array = new int[] { 0, 1 };
				break;
			case 1:
				array = new int[] { 0 };
				break;
			case 2:
				array = new int[] { 1 };
		}

		for (int i = 0; i < array.length; i++) {
			final int ident = array[i];
			if (!connectionFail[ident]) {
				connectionFail[ident] = true;
				System.err.println("CONNECTION FAIL " + (((ident + 1) % 2) + 1) + ">" + (ident + 1));
				failTimer.schedule(new TimerTask() {

					@Override
					public void run() {
						System.err.println("CONNECTION RENEW " + (((ident + 1) % 2) + 1) + ">" + (ident + 1));
						if (connectionFail[ident]) {
							connectionFail[ident] = false;
						}
					}
				}, duration);
			}
		}
		if (duration > 5000) {
			failTimer.schedule(new TimerTask() {

				@Override
				public void run() {
					if (failType == 0 || failType == 1) {
						System.err.println("MC1 closed!");
						MessageConnector tmp = mc1;
						mc1 = new MessageConnector(line1, new MessageConnectionOwner() {

							@Override
							public void incommingMessage(MessageRecord mr) {
								if (records.containsKey(mr)) {
									long t = System.currentTimeMillis() - records.get(mr);
									long b = mr.getPacket().getSize();
									records.remove(mr);
									System.out.println(String.format("CORRECT: %dB, %dms, %.2fkB/s", b, t, (b / 1024f) / (t / 1000f)));
									correct++;
									bytes += b;
									time += t;
								} else {
									System.out.println("FAILED");
								}
							}
						}, adr1, adr2);
						mc1.connect();
						tmp.close();
					}
					if (failType == 0 || failType == 2) {
						System.err.println("MC2 closed!");
						MessageConnector tmp = mc2;
						mc2 = new MessageConnector(line2, new MessageConnectionOwner() {

							@Override
							public void incommingMessage(MessageRecord mr) {
								if (records.containsKey(mr)) {
									long t = System.currentTimeMillis() - records.get(mr);
									long b = mr.getPacket().getSize();
									records.remove(mr);
									System.out.println(String.format("CORRECT: %dB, %dms, %.2fkB/s", b, t, (b / 1024f) / (t / 1000f)));
									correct++;
									bytes += b;
									time += t;
								} else {
									System.out.println("FAILED");
								}
							}
						}, adr2, adr1);
						mc2.connect();
						tmp.close();
					}

				}
			}, 5000);

		}

		failTask = new TimerTask() {

			@Override
			public void run() {
				commFailure();
			}
		};
		// Next connection fail
		failTimer.schedule(failTask, delay);
	}

	protected static int expired;

	protected static int failed;

	protected static int sent;

	protected static int other;

	protected static int newState;

	protected static int sending;

	protected static void autoSending(int[] aArg) {
		int minPeriod = aArg[0], maxPeriod = aArg[1], minSize = aArg[2], maxSize = aArg[3];
		int mode = aArg[4];
		if (maxSize < minSize) {
			maxSize = aArg[2];
			minSize = aArg[3];
		}
		int sizeSplit = maxSize - minSize;
		if (maxPeriod < minPeriod) {
			maxPeriod = aArg[0];
			minPeriod = aArg[1];
		}
		int periodSplit = maxPeriod - minPeriod;
		expired = 0;
		failed = 0;
		sent = 0;
		newState = 0;
		sending = 0;
		other = 0;
		correct = 0;
		records.clear();

		bytes = 0;
		time = 0;
		MessageRecordHandler handler = new MessageRecordHandler() {

			@Override
			public void handleMessageRecord(MessageRecord mr, MessageRecord.State state) {
				switch (state) {
					case EXPIRED:
						expired++;
						records.remove(mr);
						break;
					case FAILED:
						failed++;
						records.remove(mr);
						break;
					case SENT:
						sent++;
						break;
					case NEW:
						newState++;
						break;
					case SENDING:
						records.put(mr, System.currentTimeMillis());
						sending++;
						break;
					default:
						other++;
				}

			}
		};
		mc1.connect();
		mc2.connect();
		while (!Thread.interrupted()) {
			int size = minSize + r.nextInt(sizeSplit);
			byte[] data = new byte[size];
			byte[] data2;
			r.nextBytes(data);
			Packet p = new Packet(data);
			Packet p2;
			MessageRecord mr;
			MessageRecord mr2;
			switch (mode) {
				case 0:
					// Single mode
					mr = new MessageRecord(p, adr2, adr1, ((int) Math.floor(((double) size) / 1.024)));
					mr.addHandler(handler);
					mc1.sendMessage(mr);
					records.put(mr, mr.getEnqueueTime());
					break;
				case 1:
					// Half duplex mode
					if (r.nextBoolean()) {
						mr = new MessageRecord(p, adr2, adr1, ((int) Math.floor(((double) size) / 1.024)));
						mr.addHandler(handler);
						mc1.sendMessage(mr);
					} else {
						mr = new MessageRecord(p, adr1, adr2, ((int) Math.floor(((double) size) / 1.024)));
						mr.addHandler(handler);
						mc2.sendMessage(mr);
					}
					records.put(mr, mr.getEnqueueTime());
					break;
				case 2:
					// Full duplex mode
					data2 = new byte[size];
					r.nextBytes(data2);
					p2 = new Packet(data2);
					mr = new MessageRecord(p, adr2, adr1, ((int) Math.floor(((double) size) / 1.024)));
					mr2 = new MessageRecord(p2, adr1, adr2, ((int) Math.floor(((double) size) / 1.024)));
					mr.addHandler(handler);
					mr2.addHandler(handler);
					mc1.sendMessage(mr);
					mc2.sendMessage(mr2);
					records.put(mr, mr.getEnqueueTime());
					records.put(mr2, mr2.getEnqueueTime());
					break;
				case 3:
					// Full duplex async mode
					int size2 = minSize + r.nextInt(sizeSplit);
					data2 = new byte[size2];
					r.nextBytes(data2);
					p2 = new Packet(data2);
					mr = new MessageRecord(p, adr2, adr1, ((int) Math.floor(((double) size) / 1.024)));
					mr2 = new MessageRecord(p2, adr1, adr2, ((int) Math.floor(((double) size2) / 1.024)));
					mr.addHandler(handler);
					mr2.addHandler(handler);
					mc1.sendMessage(mr);
					mc2.sendMessage(mr2);
					records.put(mr, mr.getEnqueueTime());
					records.put(mr2, mr2.getEnqueueTime());
					break;
			}
			try {
				Thread.sleep(r.nextInt(periodSplit) + minPeriod);
			} catch (InterruptedException e) {
				break;
			}
			// if(r.nextDouble() > 0.5f) {
			// System.err.println("Closed mc1");
			// mc1.close();
			// }
		}

		try {
			Thread.sleep(5000);
		} catch (InterruptedException e) {
		}
		mc1.close();
		mc2.close();
		// Stats
		System.out.println(String.format("Statistics:\nExpired: %d\nFailed: %d\nSent: %d\nNew: %d\nSending: %d\nOther: %d", expired, failed, sent, newState,
				sending, other));
		System.out.println("Speed:" + (((double) bytes) / time) + "kB/s\n CORRECT: " + correct + "Rest:" + records.size());
	}
}
