package atc.safefly.communication.test;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.text.DateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Random;
import java.util.Scanner;

import java.nio.ByteBuffer;

import atc.safefly.communication.common.packet.Packet;
import atc.safefly.communication.common.packet.PacketConnectionOwner;
import atc.safefly.communication.common.packet.PacketInterface;
import atc.safefly.communication.common.serial.SocketSerialConnector;
import atc.safefly.communication.common.serial.UniversalSerialConnector;
import atc.safefly.communication.modem.DataPacketConnector;
import atc.safefly.communication.modem.MicrohardDiagnosticPacketConnector;

public class ModemResponseTest {

	private static BufferedWriter logFile;

	public static byte[] data = new byte[200];
	public static int speedWindow = 10000;
	public static final int timeOut = 100;// ms
	public static Random r = new Random();

	// Requested parametrs: RSSI
	public static final int[] parametrsID = new int[] { 105 };
	public static int slaveUA = 0;

	public static MicrohardDiagnosticPacketConnector diag;
	public static DataPacketConnector slave;
	public static DataPacketConnector master;
	public static final int slavePort = 5553;
	public static InetAddress masterAdr = null;

	public static HashMap<String, Long> packetCache;
	public static boolean indicator = false;
	static {
		try {
			masterAdr = InetAddress.getByAddress(new byte[] { (byte) 192, (byte) 168, 2, (byte) 155 });
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}
	}

	public static LinkedList<Long> stampsQueue = new LinkedList<Long>();

	public static int ID = 0;
	public static final int maxID = 255;

	public static final Object lock = new Object();

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length > 0) {
			if (args[0].equalsIgnoreCase("m")) {
				Thread masterThread = new Thread(new Runnable() {

					@Override
					public void run() {
						startMaster();
					}
				});
				masterThread.start();
			} else if (args[0].equalsIgnoreCase("s")) {
				final String slaveSerial = args[1];
				final int slaveBaudRate;
				try {
					slaveBaudRate = Integer.parseInt(args[2]);
				} catch (NumberFormatException e) {
					System.out.println("Parameters: s /dev/ttyO1 baudRate");
					return;
				}
				Thread slaveThread = new Thread(new Runnable() {

					@Override
					public void run() {
						startSlave(slaveSerial, slaveBaudRate);
					}
				});
				slaveThread.start();
			} else if (args[0].equalsIgnoreCase("t")) {

				Thread slaveThread = new Thread(new Runnable() {

					@Override
					public void run() {
						startTest();
					}
				});
				slaveThread.start();
			} else {
				System.out.println("Argument M(aster), S(lave) or T(No data rssi test)");
				return;
			}
		}
	}

	private static void startTest() {
		// DIAGNOSTIC module

		diag = new MicrohardDiagnosticPacketConnector(new SocketSerialConnector(masterAdr, 10001));
		if (!diag.connect()) {
			System.err.println("Diagnostic connection failed!");
			return;
		}
		// Get slave UA
		slaveUA = resolveSlaveUA();
		if (slaveUA == 0) {
			System.out.println("No slave unit found!");
			diag.dispose();
			return;
		}
		System.out.println("Slave UA=" + slaveUA);

		Thread rssiRead = new Thread(new Runnable() {

			@Override
			public void run() {
				while (!Thread.interrupted()) {
					byte[] slaveLog = diag.getParametrsByID(parametrsID, slaveUA);
					byte[] masterLog = diag.getParametrsByID(parametrsID, 1);
					StringBuilder sb = new StringBuilder();
					sb.append("1\t");
					if (masterLog == null) {
						sb.append("-\t");
					} else {
						sb.append((masterLog[4] & 0xFF) + "\t");
					}
					if (slaveLog == null) {
						sb.append("-\t");
					} else {
						sb.append((slaveLog[4] & 0xFF) + "\t");
					}
					System.out.println(sb.toString());
					ModemResponseTest.log(sb.toString());
					try {
						Thread.sleep(timeOut);
					} catch (InterruptedException e) {
						return;
					}
				}
			}
		});
		// INIT log
		initiateLogFiles();
		rssiRead.start();
		Scanner sc = new Scanner(System.in);
		while (!sc.nextLine().equalsIgnoreCase("q")) {
			System.out.println("Type 'Q' to quit!");
		}
		sc.close();
		rssiRead.interrupt();
		try {
			rssiRead.join();
		} catch (InterruptedException e) {
		}
		diag.dispose();
		disposeLog();

	}

	private static void startSlave(String serialName, int baudRate) {
		// SLAVE module
		PacketConnectionOwner slaveHandler = new PacketConnectionOwner() {

			@Override
			public void incommingPacket(PacketInterface bdp) {
				byte[] packet = bdp.getData();
				int newID = packet[0] & 0xFF;
				int shift = ((newID - ID - 1) % maxID);
				ID = newID;
				System.out.println(">>>" + ID + "[" + shift + "]");
				packet[1] = (byte) (shift + 1);
				respond(packet);
			}
		};
		slave = new DataPacketConnector(new UniversalSerialConnector(serialName, baudRate), slaveHandler);

		// CONNECTing
		if (!slave.connect()) {
			System.err.println("Slave connection failed!");
			return;
		}

		Scanner sc = new Scanner(System.in);
		while (!sc.nextLine().equalsIgnoreCase("q")) {
			System.out.println("Type Q to exit");
		}
		sc.close();
		slave.dispose();
	}

	private static void startMaster() {
		packetCache = new HashMap<String, Long>();
		diag = new MicrohardDiagnosticPacketConnector(new SocketSerialConnector(masterAdr, 10001));
		if (!diag.connect()) {
			System.err.println("Diagnostic connection failed!");
			return;
		}
		// Get slave UA
		slaveUA = resolveSlaveUA();
		if (slaveUA == 0) {
			System.out.println("No slave unit found!");
			diag.dispose();
			return;
		}
		System.out.println("Slave UA=" + slaveUA);

		// INIT log
		initiateLogFiles();

		// MASTER module
		PacketConnectionOwner masterHandler = new PacketConnectionOwner() {

			@Override
			public void incommingPacket(PacketInterface bdp) {
				synchronized (lock) {
					byte[] packet = bdp.getData();
					long currentTime = System.currentTimeMillis();
					int shift = (packet[1] & 0xFF) - 1;
					packet[1] = 0;

					byte[] slaveLog = diag.getParametrsByID(parametrsID, slaveUA);
					byte[] masterLog = diag.getParametrsByID(parametrsID, 1);
					int mRssi = 0;
					if (masterLog != null) {
						mRssi = masterLog[4] & 0xFF;
					}
					int sRssi = 0;
					if (slaveLog != null) {
						sRssi = slaveLog[4] & 0xFF;
					}
					Long timestamp = packetCache.remove(new String(packet));
					if (timestamp != null) {
						long rtt = currentTime - timestamp;
						// Clean stamps queue
						currentTime -= speedWindow;
						while (!stampsQueue.isEmpty() && stampsQueue.getFirst() < currentTime) {
							stampsQueue.removeFirst();
						}
						double speed = (2 * data.length * stampsQueue.size()) / (speedWindow / 1000f);
						StringBuilder sb = new StringBuilder();
						sb.append("1\t");
						sb.append((packet[0] & 0xFF) + "\t");
						sb.append(shift + "\t");
						sb.append(mRssi + "\t");
						sb.append(sRssi + "\t");
						sb.append(rtt + "\t");
						sb.append(String.format("%.0f Bps", speed));
						System.out.println(sb.toString());
						ModemResponseTest.log(sb.toString());

					} else {
						StringBuilder sb = new StringBuilder();
						sb.append("0\t");
						sb.append((packet[0] & 0xFF) + "\t");
						sb.append(-1 + "\t");
						sb.append(mRssi + "\t");
						sb.append(sRssi + "\t");
						sb.append("0 Bps");
						System.out.println(sb.toString());
						ModemResponseTest.log(sb.toString());
					}
					indicator = true;
					lock.notifyAll();

				}
			}
		};
		master = new DataPacketConnector(new SocketSerialConnector(masterAdr, 10002), masterHandler);
		if (!master.connect()) {
			System.err.println("Master connection failed!");
			return;
		}

		Thread sender = new Thread(new Runnable() {

			@Override
			public void run() {
				while (!Thread.interrupted()) {
					synchronized (lock) {
						indicator = false;
						data = generateData();
						Long time = System.currentTimeMillis();
						packetCache.put(new String(data), time);
						stampsQueue.addLast(time);
						master.sendPacket(new Packet(data));
						System.out.println(">>>\t" + (data[0] & 0xFF));
						try {
							lock.wait(timeOut);
						} catch (InterruptedException e) {
							break;
						}
						if (!indicator) {
							int shift = -1;
							byte[] slaveLog = diag.getParametrsByID(parametrsID, slaveUA);
							byte[] masterLog = diag.getParametrsByID(parametrsID, 1);
							int mRssi = 0;
							if (masterLog != null) {
								mRssi = masterLog[4] & 0xFF;
							}
							int sRssi = 0;
							if (slaveLog != null) {
								sRssi = slaveLog[4] & 0xFF;
							}
							StringBuilder sb = new StringBuilder();
							sb.append("-2\t");
							sb.append((data[0] & 0xFF) + "\t");
							sb.append(shift + "\t");
							sb.append(mRssi + "\t");
							sb.append(sRssi + "\t");
							sb.append("0 Bps");
							System.out.println(sb.toString());
							ModemResponseTest.log(sb.toString());
						}

					}
				}

			}
		});

		sender.start();
		Scanner sc = new Scanner(System.in);
		while (!sc.nextLine().equalsIgnoreCase("q")) {
			System.out.println("Type Q to exit");
		}
		sender.interrupt();
		try {
			Thread.sleep(speedWindow);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		System.out.println("Undelivered: " + packetCache.size());
		master.dispose();
		diag.dispose();
		disposeLog();
		sc.close();
	}

	public static int resolveSlaveUA() {
		int slaveUA = 0;
		System.out.println("Reading network configurtion...");
		byte[] config = diag.getNetworkConfiguration();
		while (config == null || config.length < 9) {
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				return 0;
			}
			System.out.println("Reading network configurtion...");
			config = diag.getNetworkConfiguration();
		}
		ByteBuffer bb = ByteBuffer.allocate(2);
		int[] slaveUAs = new int[(config.length - 9) / 2];
		for (int i = 7; i < config.length - 2; i += 2) {
			bb.put(0, config[i]);
			bb.put(1, config[i + 1]);
			slaveUAs[(i - 7) / 2] = bb.getShort(0) & 0xFFFF;
		}
		if (slaveUAs.length > 1) {
			System.out.print("Choose one of slaves: ");
			for (int i = 0; i < slaveUAs.length; i++) {
				System.out.print(slaveUAs[i] + ", ");
			}
			System.out.println();

		} else {
			return slaveUAs[0];
		}
		String l;
		Scanner sc = new Scanner(System.in);
		while ((l = sc.nextLine()) != null) {
			int posUA = 0;
			try {
				posUA = Integer.parseInt(l);
			} catch (NumberFormatException e) {
				System.out.println("Wrong: Type correct number from above");
				continue;
			}
			for (int i = 0; i < slaveUAs.length; i++) {
				if (posUA == slaveUAs[i]) {
					slaveUA = posUA;
					break;
				}
			}
			if (slaveUA == posUA) {
				break;
			}
			System.out.println("Wrong: Type correct number from above");
		}
		sc.close();
		return slaveUA;
	}

	private static byte[] generateData() {
		byte[] newData = new byte[data.length];
		r.nextBytes(newData);

		ID = (ID % maxID) + 1;
		data[0] = (byte) ID;
		data[1] = 0;
		return data;
	}

	private static void respond(byte[] packet) {
		slave.sendPacket(new Packet(packet));
	}

	private static void disposeLog() {
		synchronized (logFile) {
			try {
				logFile.flush();
				logFile.close();
				logFile = null;
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	private static void initiateLogFiles() {
		// creating logs directory
		File dir = new File("zaon_test_app_log_files");
		if (!dir.exists()) {
			System.out.println("creating logs directory: " + dir.getAbsolutePath());
			boolean result = dir.mkdir();
			if (result) {
				System.out.println("Directory created");
			}
			System.out.println();
		}
		// creating logfiles
		try {
			int i = 1;
			File file = new File(dir + "/modem_log(" + i + ").txt");
			while (file.exists()) {
				i++;
				file = new File(dir + "/modem_log(" + i + ").txt");
			}
			logFile = new BufferedWriter(new FileWriter(file));

			System.out.printf("Created log file     > %s%n", file);
		} catch (IOException ex1) {
			System.err.println("ZaonTestApp:Cannot create log file: " + ex1);
		}
	}

	private static void log(String log) {
		synchronized (logFile) {

			if (logFile != null) {
				try {
					String time = DateFormat.getDateTimeInstance().format(new Date());
					logFile.append(time + '\t');
					logFile.append(log);
					logFile.newLine();
					logFile.flush();
				} catch (IOException ex) {
					ex.printStackTrace();
				}
			}
		}
	}

}
