package atc.safefly.communication.test;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Scanner;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.log4j.Logger;

import atc.safefly.communication.common.SafeflyConnector;
import atc.safefly.communication.common.packet.AddressedPacket;
import atc.safefly.communication.common.packet.PacketConnectionOwner;
import atc.safefly.communication.common.packet.PacketInterface;
import atc.safefly.communication.common.packet.TraceRoutePacket;
import atc.safefly.communication.common.packet.TraceRoutePacket.Type;
import atc.safefly.communication.common.serial.SerialConnector;
import atc.safefly.communication.common.serial.SocketSerialConnector;
import atc.safefly.communication.common.serial.UniversalSerialConnector;
import atc.safefly.communication.modem.MicrohardDiagnosticPacketConnector;
import atc.safefly.communication.modem.TerminalConnector;

public class PlaneToPlaneTest {

	private static final String USAGE = "Plane to Plane testing application\nArguments:\n[M/S]  to run in master or slave mode\nFor slave additional arguments:\n\tS [COM] [BAUDRATE] [VERBOSE]\n\tExample: S COM1 230400 1";
	private static final String MASTER_USAGE = "Round Packet- [P SIZE ROUTE]\nCycle Packet- [C SIZE]\nBumerang Packet [B SIZE]\n\tExample:p 255 1 2 3 2 1\n Exit- [Q]";
	private static final Scanner sc = new Scanner(System.in);
	private static final Logger LOGER = Logger.getLogger(PlaneToPlaneTest.class);

	private static MicrohardDiagnosticPacketConnector diag;
	private static SafeflyConnector data;
	private static short address = 1;
	private static HashSet<Integer> slaves = new HashSet<Integer>();

	private static TraceRoutePacket current = new TraceRoutePacket();
	public static final int[] parametrsID = new int[] { 105 };

	private static InetAddress masterAdr = null;
	static {
		try {
			masterAdr = InetAddress.getByAddress(new byte[] { (byte) 192, (byte) 168, 2, (byte) 155 });
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}
	}

	private static boolean verbose = false;

	public static void main(String[] args) {

		if (args.length > 0) {
			if (args[0].equalsIgnoreCase("M")) {
				startMaster();
			} else if (args[0].equalsIgnoreCase("S")) {
				startSlave(args);
			} else {
				System.out.println("Wrong argument " + args[0] + "!\n" + USAGE);
			}
		} else {
			System.out.println("Not enought arguments!\n" + USAGE);
		}

		if (diag != null) {
			diag.dispose();
		}
		if (data != null) {
			data.dispose();
		}
		if (sc != null) {
			sc.close();
		}
	}

	private static void startMaster() {
		diag = new MicrohardDiagnosticPacketConnector(new SocketSerialConnector(masterAdr, 10001));
		if (!diag.connect()) {
			System.err.println("Diagnostic connection failed!");
			return;
		}
		data = new SafeflyConnector(new SocketSerialConnector(masterAdr, 10002), verbose);
		if (!data.connect()) {
			System.err.println("Data connection failed!");
			return;
		}

		final ConcurrentHashMap<TraceRoutePacket, TraceRoutePacket> packetMap = new ConcurrentHashMap<TraceRoutePacket, TraceRoutePacket>();
		data.setOwner(new PacketConnectionOwner() {

			@Override
			public void incommingPacket(PacketInterface packet) {
				if (packet instanceof TraceRoutePacket) {
					TraceRoutePacket p = (TraceRoutePacket) packet;
					if (p.getDstAddr() == 1 && p.increment()) {
						data.sendPacket(p);
						synchronized (current) {
							if (p.equals(current)) {
								current.notify();
							}
						}
					}
					packetMap.put(p, p);
					LOGER.debug(p);
				} else {
					System.err.println("Unrecognized packet received! Class" + packet.getClass().getName() + "\n" + packet);
				}
			}
		});

		try {
			int[] slaveUAs = resolveSlaveUA();
			for (int i = 0; i < slaveUAs.length; i++) {
				if (slaveUAs[i] == AddressedPacket.BROADCAST)
					continue;
				slaves.add(slaveUAs[i]);
			}
		} catch (InterruptedException e) {
			return;
		}

		Thread auto = null;
		System.out.println(MASTER_USAGE);
		String command;
		while (!(command = sc.nextLine()).equalsIgnoreCase("Q")) {
			packetMap.clear();
			String[] sCommand = command.split("\\s+");
			if (sCommand.length > 0) {
				if (sCommand[0].equalsIgnoreCase("P") && sCommand.length > 2) {
					try {
						TraceRoutePacket trp = createDirectPacket(sCommand);
						if (trp == null) {
							System.out.println(MASTER_USAGE);
							continue;
						}
						if (trp.increment()) {
							data.sendPacket(trp);
							packetMap.put(trp, trp);
						}

					} catch (NumberFormatException e) {
						System.out.println(MASTER_USAGE);
						continue;
					}
				} else if (sCommand[0].equalsIgnoreCase("C") && sCommand.length > 1) {
					try {
						TraceRoutePacket trp = createCyclePacket(sCommand);
						if (trp == null) {
							System.out.println(MASTER_USAGE);
							continue;
						}
						if (trp.increment()) {
							data.sendPacket(trp);
							packetMap.put(trp, trp);
						}
					} catch (NumberFormatException e) {
						System.out.println(MASTER_USAGE);
						continue;
					}
				} else if (sCommand[0].equalsIgnoreCase("B") && sCommand.length > 1) {
					try {
						TraceRoutePacket trp = createBumerangPacket(sCommand);
						if (trp == null) {
							System.out.println(MASTER_USAGE);
							continue;
						}
						if (trp.increment()) {
							data.sendPacket(trp);
							packetMap.put(trp, trp);
						}
					} catch (NumberFormatException e) {
						System.out.println(MASTER_USAGE);
						continue;
					}
				} else if (sCommand[0].equalsIgnoreCase("AC") && sCommand.length > 2) {
					if (auto != null && auto.isAlive()) {
						System.out.println("Another automatic sending in progress!");
						continue;
					}
					int period = 1000;
					try {
						if (sCommand.length > 3) {
							period = Integer.parseInt(sCommand[3]);
						}
					} catch (NumberFormatException e) {
						System.out.println(MASTER_USAGE);
						continue;
					}
					final TraceRoutePacket trp = createCyclePacket(sCommand);
					final int fPeriod = period;
					auto = new Thread(new Runnable() {

						@Override
						public void run() {
							while (!Thread.interrupted()) {
								TraceRoutePacket p = trp.copy();
								if (p.increment()) {
									data.sendPacket(p);
									packetMap.put(p, p);
								}
								try {
									Thread.sleep(fPeriod);
								} catch (InterruptedException e) {
									return;
								}
								logData(packetMap);
								StringBuilder sb = new StringBuilder();
								for (Iterator<Integer> iterator = slaves.iterator(); iterator.hasNext();) {
									int slaveUA = iterator.next();
									byte[] slaveLog = diag.getParametrsByID(parametrsID, slaveUA);
									if (slaveLog != null) {
										sb.append(String.format("[%d\t%d]\t", slaveUA, slaveLog[4] & 0xFF));
									}
								}

								LOGER.info(sb.toString());
							}
						}
					});
					LOGER.info("Auto Cycle started");
					auto.start();

				} else if (sCommand[0].equalsIgnoreCase("AB") && sCommand.length > 2) {
					if (auto != null && auto.isAlive()) {
						System.out.println("Another automatic sending in progress!");
						continue;
					}
					int period = 1000;
					try {
						if (sCommand.length > 3) {
							period = Integer.parseInt(sCommand[3]);
						}
					} catch (NumberFormatException e) {
						System.out.println(MASTER_USAGE);
						continue;
					}
					final TraceRoutePacket trp = createBumerangPacket(sCommand);
					final int fPeriod = period;
					auto = new Thread(new Runnable() {

						@Override
						public void run() {
							while (!Thread.interrupted()) {
								TraceRoutePacket p = trp.copy();
								if (p.increment()) {
									data.sendPacket(p);
									packetMap.put(p, p);
								}
								try {
									Thread.sleep(fPeriod);
								} catch (InterruptedException e) {
									return;
								}
								logData(packetMap);
								StringBuilder sb = new StringBuilder();
								for (Iterator<Integer> iterator = slaves.iterator(); iterator.hasNext();) {
									int slaveUA = iterator.next();
									byte[] slaveLog = diag.getParametrsByID(parametrsID, slaveUA);
									if (slaveLog != null) {
										sb.append(String.format("[%d\t%d]\t", slaveUA, slaveLog[4] & 0xFF));
									}
								}
								LOGER.info(sb.toString());
							}
						}
					});

					LOGER.info("Auto Bumerang started");
					auto.start();

				} else if (sCommand[0].equalsIgnoreCase("AD") && sCommand.length > 2) {
					if (auto != null && auto.isAlive()) {
						System.out.println("Another automatic sending in progress!");
						continue;
					}
					int period = 1000;
					try {
						if (sCommand.length > 3) {
							period = Integer.parseInt(sCommand[3]);
						}
					} catch (NumberFormatException e) {
						System.out.println(MASTER_USAGE);
						continue;
					}
					final TraceRoutePacket trp = createBumerangPacket(sCommand);
					sCommand[2] = "1";
					final TraceRoutePacket trpr = createBumerangPacket(sCommand);
					final int fPeriod = period;
					auto = new Thread(new Runnable() {

						@Override
						public void run() {
							while (!Thread.interrupted()) {
								try {
									current = trp.copy();
									synchronized (current) {
										if (current.increment()) {
											data.sendPacket(current);
											packetMap.put(current, current);
										}
										current.wait(fPeriod / 2);
									}
									current = trpr.copy();
									synchronized (current) {
										if (current.increment()) {
											data.sendPacket(current);
											packetMap.put(current, current);
										}
										current.wait(fPeriod / 2);
									}
								} catch (InterruptedException e) {
									return;
								}
								logData(packetMap);
								StringBuilder sb = new StringBuilder();
								for (Iterator<Integer> iterator = slaves.iterator(); iterator.hasNext();) {
									int slaveUA = iterator.next();
									byte[] slaveLog = diag.getParametrsByID(parametrsID, slaveUA);
									if (slaveLog != null) {
										sb.append(String.format("[%d\t%d]\t", slaveUA, slaveLog[4] & 0xFF));
									}
								}
								LOGER.info(sb.toString());

							}
						}
					});

					LOGER.info("Auto Double Bumerang started");
					auto.start();

				} else if (sCommand[0].equalsIgnoreCase("S")) {
					if (auto != null && auto.isAlive()) {
						System.out.println("Another automatic sending in progress!");
						continue;
					}
					int period = 1000;
					if (sCommand.length > 3) {
						period = Integer.parseInt(sCommand[3]);
					} else if (sCommand.length < 3) {
						System.out.println(MASTER_USAGE);
						continue;
					} else {
						period = 0;
					}
					final TraceRoutePacket packet = createCyclePacket(sCommand);
					final int fPeriod = period;
					final int count = 100;
					auto = new Thread(new Runnable() {

						@Override
						public void run() {
							while (!Thread.interrupted()) {
								for (int i = 0; i < count; i++) {
									current = packet.copy();
									synchronized (current) {
										if (current.increment()) {
											data.sendPacket(current);
											packetMap.put(current, current);
										}
										try {
											current.wait(fPeriod);
										} catch (InterruptedException e) {
											e.printStackTrace();
											return;
										}
									}
//									if (i % 5 == 0) {
//										StringBuilder sb = new StringBuilder();
//										for (Iterator<Integer> iterator = slaves.iterator(); iterator.hasNext();) {
//											int slaveUA = iterator.next();
//											byte[] slaveLog = diag.getParametrsByID(parametrsID, slaveUA);
//											if (slaveLog != null) {
//												sb.append(String.format("[%d\t%d]\t", slaveUA, slaveLog[4] & 0xFF));
//											}
//										}
//										LOGER.info(sb);
//									}
								}
								try {
									Thread.sleep(1000);
								} catch (InterruptedException e) {
									e.printStackTrace();
									return;
								}
								logSpeed(packetMap);
							}
						}
					});

					LOGER.info("Speed test started...");
					auto.start();

				} else if (sCommand[0].equalsIgnoreCase("I")) {
					LOGER.info("Interrupting...");
					if (auto != null && auto.isAlive()) {
						auto.interrupt();
						System.out.println("Auto thread interrupted!");
					} else {
						System.out.println("Nothing to interrupt!");
					}
				} else {
					System.out.println(MASTER_USAGE);
				}
			} else {
				System.out.println(MASTER_USAGE);
			}
		}
		if (auto != null) {
			auto.interrupt();
		}

	}

	protected static void logData(ConcurrentHashMap<TraceRoutePacket, TraceRoutePacket> packetMap) {
		for (Iterator<TraceRoutePacket> iterator = packetMap.values().iterator(); iterator.hasNext();) {
			TraceRoutePacket p = iterator.next();
			LOGER.info(p.toStats());
		}
		packetMap.clear();
	}

	protected static void logSpeed(ConcurrentHashMap<TraceRoutePacket, TraceRoutePacket> packetMap) {
		if (packetMap.isEmpty()) {
			return;
		}
		long start = Long.MAX_VALUE;
		long stop = 0;
		int fail = 0;
		int count = 0;
		TraceRoutePacket p = null;
		long[] time = new long[0];
		for (Iterator<TraceRoutePacket> iterator = packetMap.values().iterator(); iterator.hasNext();) {
			p = iterator.next();
			time = p.getTime();

			if (time[0] < start)
				start = time[0];
			if (time[time.length - 1] > stop)
				stop = time[time.length - 1];
			for (int i = 0; i < time.length; i++) {
				if (time[i] == 0) {
					fail++;
					break;
				}
				if (i > 0)
					count++;
			}
		}
		int traceSize = time.length - 1;
		double complete = count / (double) (traceSize * packetMap.size());
		double duration = (stop - start) / 1000f;
		double speed = (p.getSize() * count) / duration;
		String log = String.format("Fail:%d Count:%d Time:%.2f Complete:%.3f Speed:%.2f", fail, count, duration, complete, speed);
		LOGER.info(log);
		packetMap.clear();
	}

	private static void startSlave(String[] args) {
		if (args.length > 3) {
			try {
				if (Integer.parseInt(args[3]) > 0) {
					verbose = true;
				}
			} catch (NumberFormatException e) {
			}
		}
		if (args.length > 2) {
			try {
				SerialConnector subConnector = new UniversalSerialConnector(args[1], Integer.parseInt(args[2]));

				TerminalConnector tc = new TerminalConnector(subConnector);
				if (tc.connect()) {
					address = (short) tc.getAddress();
					LOGER.info("Current unit address is " + (address & 0xFFFF));
					tc.close();
				} else {
					System.err.println("Failed to get modem unit address!");
					return;
				}

				data = new SafeflyConnector(subConnector, verbose);
				if (!data.connect()) {
					System.err.println("Data connection failed!");
					return;
				}
				data.setOwner(new PacketConnectionOwner() {

					@Override
					public void incommingPacket(PacketInterface packet) {
						if (packet instanceof TraceRoutePacket) {
							processSlavePacket((TraceRoutePacket) packet);
						} else {
							System.err.println("Unrecognized packet received! Class" + packet.getClass().getName() + "\n" + packet);
						}
					}
				});
			} catch (NumberFormatException e) {
				System.out.println("Inavlid arguments for slave mode!\n" + USAGE);
				return;
			}

			String line;
			while (!(line = sc.nextLine()).equalsIgnoreCase("Q")) {
				System.out.println("Type 'Q' to quit!");
			}
		} else {
			System.out.println("Inavlid arguments for slave mode!\n" + USAGE);
		}
	}

	protected static void processSlavePacket(TraceRoutePacket packet) {
		if (packet.getDstAddr() == address) {
			if (packet.increment()) {
				data.sendPacket(packet);
			}
			LOGER.info(packet.toStats());
		} else {
			LOGER.debug(packet.toStats());
		}
	}

	private static TraceRoutePacket createDirectPacket(String[] sCommand) throws IllegalArgumentException {
		int size = 0;
		try {
			size = Integer.parseInt(sCommand[1]);
		} catch (NumberFormatException e) {
			throw new IllegalArgumentException(e.getMessage());
		}

		short[] address = new short[sCommand.length - 2];
		boolean parseFail = false;
		for (int i = 0; i < address.length; i++) {
			address[i] = (short) Integer.parseInt(sCommand[i + 2]);
			if (address[i] != 1 && !slaves.contains(address[i] & 0xFFFF)) {
				System.out.println("Slave " + address[i] + " is not availible!");
				parseFail = true;
			}
		}
		if (parseFail) {
			return null;
		}
		return new TraceRoutePacket(address, size, Type.DIRECT);
	}

	private static TraceRoutePacket createBumerangPacket(String[] sCommand) throws IllegalArgumentException {
		int size = 0;
		boolean reverse = false;
		try {
			size = Integer.parseInt(sCommand[1]);
			if (sCommand.length > 2) {
				reverse = Integer.parseInt(sCommand[2]) > 0;
			}
		} catch (NumberFormatException e) {
			throw new IllegalArgumentException(e.getMessage());
		}
		short[] route = new short[slaves.size() * 2 + 1];
		route[0] = 1;
		route[route.length - 1] = 1;
		int i = 1;
		int j = route.length - 2;
		if (reverse) {
			i = (route.length / 2);
			j = i;
		}

		for (Iterator<Integer> iterator = slaves.iterator(); iterator.hasNext();) {
			int addr = iterator.next();
			route[i] = (short) addr;
			route[j] = (short) addr;
			i++;
			j--;
		}
		return new TraceRoutePacket(route, size, Type.BUMERANG);
	}

	private static TraceRoutePacket createCyclePacket(String[] sCommand) {
		int size = 0;
		boolean reverse = false;
		try {
			size = Integer.parseInt(sCommand[1]);
			if (sCommand.length > 2) {
				reverse = Integer.parseInt(sCommand[2]) > 0;
			}
		} catch (NumberFormatException e) {
			throw new IllegalArgumentException(e.getMessage());
		}
		short[] route = new short[slaves.size() + 2];
		route[0] = 1;
		route[route.length - 1] = 1;
		if (reverse) {
			int i = route.length - 2;
			for (Iterator<Integer> iterator = slaves.iterator(); iterator.hasNext();) {
				int addr = iterator.next();
				route[i] = (short) addr;
				i--;
			}
		} else {
			int i = 1;
			for (Iterator<Integer> iterator = slaves.iterator(); iterator.hasNext();) {
				int addr = iterator.next();
				route[i] = (short) addr;
				i++;
			}
		}
		return new TraceRoutePacket(route, size, Type.CYCLE);
	}

	private static int[] resolveSlaveUA() throws InterruptedException {
		String l;
		int[] slaveUAs;
		do {
			System.out.println("Reading network configurtion...");
			byte[] config = diag.getNetworkConfiguration();
			while (config == null || config.length < 9) {
				Thread.sleep(1000);
				System.out.println("Reading network configurtion...");
				config = diag.getNetworkConfiguration();
			}

			ByteBuffer bb = ByteBuffer.allocate(2);
			slaveUAs = new int[(config.length - 9) / 2];
			for (int i = 7; i < config.length - 2; i += 2) {
				bb.put(0, config[i]);
				bb.put(1, config[i + 1]);
				slaveUAs[(i - 7) / 2] = bb.getShort(0) & 0xFFFF;
			}
			System.out.print("Slaves found: ");
			for (int i = 0; i < slaveUAs.length; i++) {
				System.out.print(slaveUAs[i] + ", ");
			}
			System.out.println("\n Continue finding? [Y]/N");

			l = sc.nextLine();
		} while (!l.equalsIgnoreCase("N"));
		return slaveUAs;
	}
}
