package atc.safefly.communication.utils;

import org.apache.log4j.Logger;


public class ZaonData {
	private static final int BIT_MASK = 0x0003;
	private static final Logger LOGGER = Logger.getLogger(ZaonData.class);

	public enum Status {
		UNKNOWN, BELOW, ABOVE, SAME_LEV
	}

	public enum Change {
		NO, CLIMB, DESCEND
	}

	private TrafficData[] aircrafts;
	private int decodedAlt;
	private int squawkCode;

	public ZaonData() {
		aircrafts = new TrafficData[3];
		for (int i = 0; i < aircrafts.length; i++) {
			aircrafts[i] = new TrafficData();
		}
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append(String.format("Zaon XRX: Own info [Decoded Altitude: %dm\tSquawkCode: %d]\n", decodedAlt, squawkCode));
		for (int i = 0; i < aircrafts.length; i++) {
			sb.append("PA");
			sb.append(i + 1);
			sb.append(" => ");
			sb.append(aircrafts[i].toString());
			sb.append('\n');
		}
		return sb.toString();
	}

	public void processData(int chnlId, int value) {
		if (chnlId > 4369) {
			// Common data
			switch (chnlId) {
				case 4370: // PA Flags
					LOGGER.debug("PA Flags: " + Integer.toBinaryString(value));
					setPaFlags(value);
					break;
				case 4371: // Own decoded altitude
					decodedAlt = value;
					break;
				case 4372: // Own squawk code
					squawkCode = value;
					break;
			}
		} else {
			// Priority specific data
			chnlId -= 4358;//Lower possible ID
			int paNr = chnlId / 4;
			int featureNr = chnlId % 4;
			switch (featureNr) {
				case 0:
					aircrafts[paNr].range = value;
					break;
				case 1:
					aircrafts[paNr].decodedAlt = value;
					break;
				case 2:
					aircrafts[paNr].relativeAlt = value;
					break;
				case 3:
					aircrafts[paNr].magBearing = value;
					break;
			}
		}
	}

	private void setPaFlags(int value) {
		for (int i = 0; i < aircrafts.length; i++) {
			int statusId = value & BIT_MASK;
			int changeId = (value >>> 4) & BIT_MASK;
			if (statusId > -1 && statusId < Status.values().length) {
				aircrafts[i].status = Status.values()[statusId];
			} else {
				LOGGER.debug("Status PA" + (i + 1) + " out of range: " + statusId);
			}
			if (changeId > -1 && changeId < Change.values().length) {
				aircrafts[i].change = Change.values()[changeId];
			} else {
				LOGGER.debug("Change PA" + (i + 1) + " out of range: " + changeId);
			}

			value = value >>> 8;
		}
	}

	public class TrafficData {

		public TrafficData() {
			range = 0;
			decodedAlt = 0;
			relativeAlt = 0;
			magBearing = 0;

			status = Status.UNKNOWN;
			change = Change.NO;
		}

		@Override
		public String toString() {
			return String.format("Status: %s\tChange:%s\tRange: %dm\tDecoded Altitude: %dm\tRelative Altitude %dm\tMagneticBearing: %d", status.name(),
					change.name(), range, decodedAlt, relativeAlt, magBearing);
		}

		public int range;
		public int decodedAlt;
		public int relativeAlt;
		public int magBearing;

		public Status status;
		public Change change;

	}
}
