package atc.safefly.communication.utils;

import java.util.Arrays;

import atc.procerus.communication.ProcerusPacket;
import atc.procerus.utils.ByteUtils;
import atc.safefly.communication.common.packet.ApplicationPacket;
import atc.safefly.utils.SafeflyConstants;

/**
 * @author agent
 * 
 */
public class ZaonPacket implements ProcerusPacket {

	private class TrafficAircraft {

		private double range;
		private int decodedAltitude;
		private int relativeAltitude;
		private int magneticBearing;

		private boolean above; // true == above, false == below
		private boolean climbing;
		private boolean descending;

		// getters & setters
		/**
		 * 
		 * @return range in NM
		 */
		public double getRange() {
			return range;
		}

		/**
		 * 
		 * @param range
		 *            distance to the aricraft in NM
		 */
		public void setRange(double range) {
			this.range = range;
		}

		/**
		 * 
		 * @return decoded altitude in feet
		 */
		public int getDecodedAltitude() {
			return decodedAltitude;
		}

		/**
		 * 
		 * @param decodedAltitude
		 *            in feet
		 */
		public void setDecodedAltitude(int decodedAltitude) {
			this.decodedAltitude = decodedAltitude;
		}

		/**
		 * 
		 * @return relative altitude in feet
		 */
		public int getRelativeAltitude() {
			return relativeAltitude;
		}

		/**
		 * 
		 * @param relativeAltitude
		 *            in feet
		 */
		public void setRelativeAltitude(int relativeAltitude) {
			this.relativeAltitude = relativeAltitude;
		}

		/**
		 * 
		 * @return magnetic bearing degrees
		 */
		public int getMagneticBearing() {
			return magneticBearing;
		}

		/**
		 * 
		 * @param magneticBearing
		 *            in degrees
		 */
		public void setMagneticBearing(int magneticBearing) {
			this.magneticBearing = magneticBearing;
		}

		/**
		 * 
		 * @return true if aircraft is above
		 */
		public boolean isAbove() {
			return above;
		}

		/**
		 * 
		 * @param above
		 *            true for aircraft above host, false otherwise
		 */
		public void setAbove(boolean above) {
			this.above = above;
		}

		/**
		 * 
		 * @return true if detected aircraft is climbing
		 */
		public boolean isClimbing() {
			return climbing;
		}

		/**
		 * Sets vertical trend indicator climbing
		 * 
		 * @param climbing
		 */
		public void setClimbing(boolean climbing) {
			this.climbing = climbing;
		}

		/**
		 * 
		 * @return true if detected aircraft is descending
		 */
		public boolean isDescending() {
			return descending;
		}

		/**
		 * Sets vertical trend indicator for descending
		 * 
		 * @param descending
		 */
		public void setDescending(boolean descending) {
			this.descending = descending;
		}
	}

	// variables
	/** packet flags, unsigned char */
	public byte packetFlags;

	/** application packet */
	public ApplicationPacket appPacket;

	private final int ZAON_PACKET_LENGTH = 22;
	// host properties
	/** altitude of host aircraft in feet */
	private int pressureAltitudeFT;
	private int squawkCode;
	private String digitA;
	private String digitB;
	private String digitC;
	private String digitD;
	// packet variables
	private byte verificationByte1;
	private byte verificationByte2;
	private byte checksum;
	private byte checksumCounter;
	// traffic
	private TrafficAircraft traffic1;
	private TrafficAircraft traffic2;
	private TrafficAircraft traffic3;
	// whole packet
	byte[] packetBytes = new byte[ZAON_PACKET_LENGTH];
	// temporary variables
	private byte helpByte;

	// end of variables declaration

	// getters
	public ZaonPacket() {
		this.traffic1 = new TrafficAircraft();
		this.traffic2 = new TrafficAircraft();
		this.traffic3 = new TrafficAircraft();
	}

	/**
	 * @return pressured altitude in feet [ft]
	 */
	public int getPressureAltitudeFT() {
		return pressureAltitudeFT;
	}

	/**
	 * @return host squawk code
	 */
	public int getSquawkCode() {
		return squawkCode;
	}
	
	//traffic getters
	
	//trafic 1 getters
	
	public double getTraffic1Range(){
		return this.traffic1.getRange();
	}
	public int getTraffic1DecodedAltitude(){
		return this.traffic1.getDecodedAltitude();
	}
	public int getTraffic1RelativeAltitude(){
		return this.traffic1.getRelativeAltitude();
	}
	public int getTraffic1MagneticBearing(){
		return this.traffic1.getMagneticBearing();
	}
	public boolean isTraffic1Above(){
		return this.traffic1.isAbove();
	}
	public boolean isTraffic1Climbing(){
		return this.traffic1.isClimbing();
	}
	public boolean isTraffic1Descending(){
		return this.traffic1.isAbove();
	}
	//traffic2 getters
	public double getTraffic2Range(){
		return this.traffic2.getRange();
	}
	public int getTraffic2DecodedAltitude(){
		return this.traffic2.getDecodedAltitude();
	}
	public int getTraffic2RelativeAltitude(){
		return this.traffic2.getRelativeAltitude();
	}
	public int getTraffic2MagneticBearing(){
		return this.traffic2.getMagneticBearing();
	}
	public boolean isTraffic2Above(){
		return this.traffic2.isAbove();
	}
	public boolean isTraffic2Climbing(){
		return this.traffic2.isClimbing();
	}
	public boolean isTraffic2Descending(){
		return this.traffic2.isAbove();
	}
	// traffic 3 getters
	public double getTraffic3Range(){
		return this.traffic3.getRange();
	}
	public int getTraffic3DecodedAltitude(){
		return this.traffic3.getDecodedAltitude();
	}
	public int getTraffic3RelativeAltitude(){
		return this.traffic3.getRelativeAltitude();
	}
	public int getTraffic3MagneticBearing(){
		return this.traffic3.getMagneticBearing();
	}
	public boolean isTraffic3Above(){
		return this.traffic3.isAbove();
	}
	public boolean isTraffic3Climbing(){
		return this.traffic3.isClimbing();
	}
	public boolean isTraffic3Descending(){
		return this.traffic3.isAbove();
	}
	

	// end of getters

	public ZaonPacket(byte[] packetBytes) {
		this();
		this.packetBytes = packetBytes;
		this.decodePacketBytes(packetBytes);

	}

	public static byte decryptByte(byte b) {
		if (b == 0x00 || b == 0x76) {
			return b;
		} else {
			return (byte) (b ^ 0x76);
		}
	}

	private static Boolean isBitSet(byte b, int bit) {
		return (b & (1 << bit)) != 0;
	}

	public String toString(boolean hex, boolean decryptedHex, boolean decryptedValues, boolean legend) {
		String result = "";
		if (hex) {
			if (legend) {
				result += "raw  data %X > ";
			}
			result += ByteUtils.byteArrayToHexString(this.packetBytes);
			result += String.format("%n");
		}
		if (decryptedHex) {
			if (legend) {
				result += "dec. data %X > ";
			}
			for (int i = 0; i < packetBytes.length - 1; i++) {
				result += String.format("%02X ", decryptByte(packetBytes[i]));
			}
			result += String.format("%02X", packetBytes[packetBytes.length - 1]);
			result += String.format("%n");
		}
		if (decryptedValues) {
			result += String.format("    Host> Pressure Altitude: %d[ft] Squawk code: %d%n", this.pressureAltitudeFT,
					this.squawkCode);
			result += String
					.format("Traffic1> Range: %.1f[NM], Decoded Altitude: %d[ft], Relative Altitude: %d[ft], Magnetic Bearing: %d[deg], Above: %b, Climbing: %b, Descending: %b%n",
							this.traffic1.getRange(), this.traffic1.getDecodedAltitude(), this.traffic1.getRelativeAltitude(),
							this.traffic1.getMagneticBearing(), this.traffic1.isAbove(), this.traffic1.isClimbing(),
							this.traffic1.isDescending());
			result += String
					.format("Traffic2> Range: %.1f[NM], Decoded Altitude: %d[ft], Relative Altitude: %d[ft], Magnetic Bearing: %d[deg], Above: %b, Climbing: %b, Descending: %b%n",
							this.traffic2.getRange(), this.traffic2.getDecodedAltitude(), this.traffic2.getRelativeAltitude(),
							this.traffic2.getMagneticBearing(), this.traffic2.isAbove(), this.traffic2.isClimbing(),
							this.traffic2.isDescending());
			result += String
					.format("Traffic3> Range: %.1f[NM], Decoded Altitude: %d[ft], Relative Altitude: %d[ft], Magnetic Bearing: %d[deg], Above: %b, Climbing: %b, Descending: %b%n",
							this.traffic3.getRange(), this.traffic3.getDecodedAltitude(), this.traffic3.getRelativeAltitude(),
							this.traffic3.getMagneticBearing(), this.traffic3.isAbove(), this.traffic3.isClimbing(),
							this.traffic3.isDescending());

		}

		return result;
	}

	/**
	 * Decode the byte array to the ZaonPacket
	 * 
	 * @param array
	 *            input byte array
	 * @return true if the Checksum succeeds and ZaonPacket has been created,
	 *         false if Checksum validation fails and ZaonPacket has been not
	 *         created.
	 */
	public boolean decodePacketBytes(byte[] array) {
		System.arraycopy(array, 0, this.packetBytes, 0, ZAON_PACKET_LENGTH);

		this.checksum = this.packetBytes[ZAON_PACKET_LENGTH - 1];
		this.checksumCounter = 0;
		for (int i = 0; i < ZAON_PACKET_LENGTH - 1; i++) {
			this.checksumCounter += this.packetBytes[i];
		}
		if (this.checksumCounter != this.checksum) {
			return false;
		}

		this.pressureAltitudeFT = decryptByte(this.packetBytes[0]) * 100;

		this.traffic1.setRange(((double) (decryptByte(this.packetBytes[1]) & 0xFF)) / 10);
		this.traffic2.setRange(((double) (decryptByte(this.packetBytes[5]) & 0xFF)) / 10);
		this.traffic3.setRange(((double) (decryptByte(this.packetBytes[9]) & 0xFF)) / 10);

		this.traffic1.setDecodedAltitude(decryptByte(this.packetBytes[2]) * 100);
		this.traffic2.setDecodedAltitude(decryptByte(this.packetBytes[6]) * 100);
		this.traffic3.setDecodedAltitude(decryptByte(this.packetBytes[10]) * 100);

		this.traffic1.setRelativeAltitude(decryptByte(this.packetBytes[3]) * 100);
		this.traffic2.setRelativeAltitude(decryptByte(this.packetBytes[7]) * 100);
		this.traffic3.setRelativeAltitude(decryptByte(this.packetBytes[11]) * 100);

		this.traffic1.setMagneticBearing(decryptByte(this.packetBytes[4]) * 10);
		this.traffic2.setMagneticBearing(decryptByte(this.packetBytes[8]) * 10);
		this.traffic3.setMagneticBearing(decryptByte(this.packetBytes[12]) * 10);

		helpByte = decryptByte(this.packetBytes[13]);
		this.traffic1.setAbove(isBitSet(helpByte, 0));
		this.traffic2.setAbove(isBitSet(helpByte, 1));
		this.traffic3.setAbove(isBitSet(helpByte, 2));

		helpByte = decryptByte(this.packetBytes[14]);
		this.traffic1.setClimbing(isBitSet(helpByte, 0));
		this.traffic2.setClimbing(isBitSet(helpByte, 2));
		this.traffic3.setClimbing(isBitSet(helpByte, 4));

		this.traffic1.setDescending(isBitSet(helpByte, 1));
		this.traffic2.setDescending(isBitSet(helpByte, 3));
		this.traffic3.setDescending(isBitSet(helpByte, 5));

		this.digitA = Integer.toString(decryptByte(this.packetBytes[15]));
		this.digitB = Integer.toString(decryptByte(this.packetBytes[16]));
		this.digitC = Integer.toString(decryptByte(this.packetBytes[17]));
		this.digitD = Integer.toString(decryptByte(this.packetBytes[18]));
		this.squawkCode = Integer.parseInt(this.digitA + this.digitB + this.digitC + this.digitD);

		this.verificationByte1 = decryptByte(this.packetBytes[19]);
		this.verificationByte2 = decryptByte(this.packetBytes[20]);

		return true;
	}

	/** Assembly packet to be sent. Generate CRCs. */
	@Override
	public byte[] assemblyPacket() {
		return packetBytes;
	}

	/** return packet's data. */
	@Override
	public byte[] getPacketAsBytes() {
		return this.assemblyPacket();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see atc.procerus.communication.ProcerusPacket#getSrcAddress()
	 */
	@Override
	public int getSrcAddress() {
		// TODO Auto-generated method stub
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see atc.procerus.communication.ProcerusPacket#getDstAddress()
	 */
	@Override
	public int getDstAddress() {
		// TODO Auto-generated method stub
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see atc.procerus.communication.ProcerusPacket#getType()
	 */
	@Override
	public byte getType() {
		// TODO Auto-generated method stub
		return 0;
	}

}
